import numpy as np
from pyniel.python_tools.timetools import WalltimeRate
import time

SIDE_SPEED = 1.
FRONT_SPEED = 1.
BACK_SPEED = 1.
ROT_SPEED = 0.4


class EnvPlayer(object):
    def __init__(self, env, render_mode="human", step_by_step=False):
        self.env = env
        self.render_mode = render_mode
        self.STEP_BY_STEP = step_by_step
        self.boost = False
        self.run()

    def key_press(self, k, mod):
        from pyglet.window import key
        if k == 0xFF0D or k == key.ESCAPE:
            self.exit = True
        if k == key.R:
            self.restart = True
        if k in [key.RIGHT, key.E]:
            self.action[2] = -ROT_SPEED
        if k in [key.LEFT, key.Q]:
            self.action[2] = +ROT_SPEED
        if k in [key.UP, key.W]:
            self.action[0] = +FRONT_SPEED
        if k in [key.DOWN, key.S]:
            self.action[0] = -BACK_SPEED
        if k in [key.D]:
            self.action[1] = -SIDE_SPEED
        if k in [key.A]:
            self.action[1] = +SIDE_SPEED
        if k in [key.LSHIFT]:
            self.boost = True
        if k in [key.SPACE]:
            pass
        self.action_key_is_set = True

    def key_release(self, k, mod):  # reverse action of pressed
        from pyglet.window import key
        if k in [key.RIGHT, key.E] and self.action[2] == -ROT_SPEED:
            self.action[2] = 0
        if k in [key.LEFT, key.Q] and self.action[2] == +ROT_SPEED:
            self.action[2] = 0
        if k in [key.UP, key.W] and self.action[0] == +FRONT_SPEED:
            self.action[0] = 0
        if k in [key.DOWN, key.S] and self.action[0] == -BACK_SPEED:
            self.action[0] = 0
        if k in [key.D] and self.action[1] == -SIDE_SPEED:
            self.action[1] = 0
        if k in [key.A] and self.action[1] == +SIDE_SPEED:
            self.action[1] = 0
        if k in [key.LSHIFT]:
            self.boost = False
        if k in [key.SPACE]:
            pass

    def reset(self):
        # reset env
        print("Resetting")
        self.env.reset()
        # reset player
        self.realtime_rate = WalltimeRate(1.0 / self.env._get_dt())
        self.action = np.array([0.0, 0.0, 0.0])
        self.restart = False
        self.exit = False
        self.action_key_is_set = False
        self.restart = False
        # reset viewer, connect callbacks
        self.env.render(mode=self.render_mode)
        self.env._get_viewer().window.on_key_press = self.key_press
        self.env._get_viewer().window.on_key_release = self.key_release

    def run(self):
        # run interactively ----------------------
        self.reset()

        print()
        print("-------------------")
        print("Running environment")
        print("press WASD to move, Q and E to rotate, shift to speed-up.")
        print("R to reset, ESC to exit.")
        print("...")

        i = 0
        while not self.exit:
            i += 1
            # synchronize (either with keypresses or walltime)
            if self.STEP_BY_STEP:
                # wait for keypress
                while True:
                    if self.boost:
                        break
                    if not self.action_key_is_set:
                        self.env.render(mode=self.render_mode)
                        time.sleep(0.01)
                    else:
                        self.action_key_is_set = False
                        break
            else:
                if not self.boost:
                    self.realtime_rate.sleep()
            # step once
            obs, rew, done, info = self.env.step(self.action)
            self.env.render(mode=self.render_mode)
            #         impglet
            #         pygage.get_buffer_manager().get_color_buffer().save("/tmp/env{:05}.png".format(i))
            if done or self.restart:
                self.reset()
        self.env.close()



import numpy as np
from gym.envs.classic_control import rendering
import pyglet
from pyglet import gl


def render_lidar(
    ranges, min_angle, max_angle, min_range=0.3, mode="human", viewer=None
):
    if mode == "human":
        # Window and viewport size
        WINDOW_W = 512
        WINDOW_H = 512
        WINDOW_H_METERS = 10.0
        RESOLUTION_M_PER_PX = WINDOW_H_METERS / WINDOW_H
        VP_W = WINDOW_W
        VP_H = WINDOW_H
        # Create viewer
        if viewer is None:
            viewer = rendering.Viewer(WINDOW_W, WINDOW_H)
            viewer.score_label = pyglet.text.Label(
                "0000",
                font_size=36,
                x=20,
                y=WINDOW_H * 2.5 / 40.00,
                anchor_x="left",
                anchor_y="center",
                color=(255, 255, 255, 255),
            )
        viewer.draw_circle(r=10, color=(0.3, 0.3, 0.3))
        win = viewer.window
        win.switch_to()
        win.dispatch_events()
        win.clear()
        gl.glViewport(0, 0, VP_W, VP_H)
        # colors
        bgcolor = np.array([0.9, 0.9, 1.0])
        lidarcolor = np.array([1.0, 0.0, 0.0])
        # Green background
        gl.glBegin(gl.GL_QUADS)
        gl.glColor4f(bgcolor[0], bgcolor[1], bgcolor[2], 1.0)
        gl.glVertex3f(0, VP_H, 0)
        gl.glVertex3f(VP_W, VP_H, 0)
        gl.glVertex3f(VP_W, 0, 0)
        gl.glVertex3f(0, 0, 0)
        gl.glEnd()
        # LIDAR
        sensor_i = WINDOW_H / 2.0
        sensor_j = WINDOW_W / 2.0
        angles = np.linspace(min_angle, max_angle, len(ranges))
        ijranges = np.array(ranges) / RESOLUTION_M_PER_PX
        for th, r in zip(angles, ijranges):
            ray_start_i = np.clip(sensor_i + min_range * np.cos(th), 1, WINDOW_W - 1)
            ray_start_j = np.clip(sensor_j + min_range * np.sin(th), 1, WINDOW_H - 1)
            ray_end_i = np.clip(sensor_i + r * np.cos(th), 1, WINDOW_W - 1)
            ray_end_j = np.clip(sensor_j + r * np.sin(th), 1, WINDOW_H - 1)
            gl.glBegin(gl.GL_LINE_LOOP)
            gl.glColor4f(lidarcolor[0], lidarcolor[1], lidarcolor[2], 0.1)
            gl.glVertex3f(ray_start_i, ray_start_j, 0)
            gl.glVertex3f(ray_end_i, ray_end_j, 0)
            gl.glEnd()
        # Text
        viewer.score_label.text = "{}m".format(WINDOW_H_METERS)
        viewer.score_label.draw()
        win.flip()
        return viewer


def render_lidar_batch(
    batch_ranges, min_angle, max_angle, min_range=0.3, mode="human", viewer=None
):
    if mode == "human":
        N_COL_WINDOWS = 10
        N_ROW_WINDOWS = 10
        N_WINDOWS = N_COL_WINDOWS * N_ROW_WINDOWS
        # Window and viewport size
        WINDOW_W = 64
        WINDOW_H = 64
        WINDOW_H_METERS = 10.0
        RESOLUTION_M_PER_PX = WINDOW_H_METERS / WINDOW_H
        VP_W = WINDOW_W * N_COL_WINDOWS
        VP_H = WINDOW_H * N_ROW_WINDOWS
        # Create viewer
        if viewer is None:
            viewer = rendering.Viewer(VP_W, VP_H)
            viewer.score_label = pyglet.text.Label(
                "0000",
                font_size=36,
                x=20,
                y=WINDOW_H * 2.5 / 40.00,
                anchor_x="left",
                anchor_y="center",
                color=(255, 255, 255, 255),
            )
        viewer.draw_circle(r=10, color=(0.3, 0.3, 0.3))
        win = viewer.window
        win.switch_to()
        win.dispatch_events()
        win.clear()
        gl.glViewport(0, 0, VP_W, VP_H)
        # colors
        bgcolor = np.array([0.9, 0.9, 1.0])
        lidarcolor = np.array([1.0, 0.0, 0.0])
        # Green background
        gl.glBegin(gl.GL_QUADS)
        gl.glColor4f(bgcolor[0], bgcolor[1], bgcolor[2], 1.0)
        gl.glVertex3f(0, VP_H, 0)
        gl.glVertex3f(VP_W, VP_H, 0)
        gl.glVertex3f(VP_W, 0, 0)
        gl.glVertex3f(0, 0, 0)
        gl.glEnd()
        # LIDAR
        assert len(batch_ranges) == 100
        for n, ranges in enumerate(batch_ranges):
            col = n % N_COL_WINDOWS
            row = n // N_COL_WINDOWS
            window_offset_i = row * WINDOW_W
            window_offset_j = col * WINDOW_H
            sensor_i = window_offset_i + WINDOW_W / 2.0
            sensor_j = window_offset_j + WINDOW_H / 2.0
            angles = np.linspace(min_angle, max_angle, len(ranges))
            ijranges = np.array(ranges) / RESOLUTION_M_PER_PX
            for th, r in zip(angles, ijranges):
                ray_start_i = np.clip(
                    sensor_i + min_range * np.cos(th),
                    window_offset_i,
                    window_offset_i + WINDOW_H - 1,
                )
                ray_start_j = np.clip(
                    sensor_j + min_range * np.sin(th),
                    window_offset_j,
                    window_offset_j + WINDOW_W - 1,
                )
                ray_end_i = np.clip(
                    sensor_i + r * np.cos(th),
                    window_offset_i,
                    window_offset_i + WINDOW_H - 1,
                )
                ray_end_j = np.clip(
                    sensor_j + r * np.sin(th),
                    window_offset_j,
                    window_offset_j + WINDOW_W - 1,
                )
                gl.glBegin(gl.GL_LINE_LOOP)
                gl.glColor4f(lidarcolor[0], lidarcolor[1], lidarcolor[2], 0.1)
                gl.glVertex3f(ray_start_i, ray_start_j, 0)
                gl.glVertex3f(ray_end_i, ray_end_j, 0)
                gl.glEnd()
        # Text
        #         viewer.score_label.text = "{}m".format(WINDOW_H_METERS)
        #         viewer.score_label.draw()
        win.flip()
        return viewer



import os
from stable_baselines import PPO2

from navrep.tools.custom_policy import CustomPolicy, Custom1DPolicy

class E2ECPolicy(object):
    """ thin wrapper for gym policies """
    def __init__(self, model_path=None, model=None):
        if model is not None:
            self.model = model
        else:
            self.model_path = model_path
            if self.model_path is None:
                self.model_path = os.path.expanduser(
                    "~/navrep/models/gym/e2enavreptrainenv_latest_PPO_ckpt")
            self.model = PPO2.load(self.model_path, policy=CustomPolicy)
            print("Model '{}' loaded".format(self.model_path))

    def act(self, obs):
        action, _states = self.model.predict(obs, deterministic=True)
        return action

class E2E1DCPolicy(object):
    """ thin wrapper for gym policies """
    def __init__(self, model_path=None, model=None):
        if model is not None:
            self.model = model
        else:
            self.model_path = model_path
            if self.model_path is None:
                self.model_path = os.path.expanduser(
                    "~/navrep/models/gym/e2e1dnavreptrainenv_latest_PPO_ckpt")
            self.model = PPO2.load(self.model_path, policy=Custom1DPolicy)
            print("Model '{}' loaded".format(self.model_path))

    def act(self, obs):
        action, _states = self.model.predict(obs, deterministic=True)
        return action



import os
import numpy as np
from pyniel.python_tools.path_tools import make_dir_if_not_exists
from stable_baselines.ppo2 import PPO2
import rl_agent.common_custom_policies  # noqa

from navrep.tools.commonargs import parse_common_args
from navrep.envs.ianenv import IANEnv
from navrep.scripts.cross_test_navreptrain_in_ianenv import run_test_episodes

# constants. If you change these, think hard about what which assumptions break.
_90 = 90  # guldenring downsampled scan size
_1080 = 1080  # navrep scan size
_540 = 1080 // 2
_8 = 8  # number of guldenrings waypoints

class GuldenringCPolicy():
    def __init__(self):
        self.model = PPO2.load(os.path.expanduser(
            "~/Code/drl_local_planner_ros_stable_baselines/example_agents/ppo2_1_raw_data_cont_0/ppo2_1_raw_data_cont_0.pkl"))  # noqa

    def act(self, obs):
        action, _state = self.model.predict(obs, deterministic=True)
        return action

class GuldenringWrapperForIANEnv(IANEnv):
    def _convert_obs(self, ianenv_obs):
        scan, robotstate = ianenv_obs
        guldenring_obs = np.zeros((1, _90 + _8 * 2, 1))
        # rotate lidar scan so that first ray is at -pi
        rotated_scan = np.zeros_like(scan)
        rotated_scan[:_540] = scan[_540:]
        rotated_scan[_540:] = scan[:_540]
        # 1080-ray to 90-ray: for each angular section we take the min of the returns
        lidar_upsampling = _1080 // _90
        downsampled_scan = rotated_scan.reshape((-1, lidar_upsampling))
        downsampled_scan = np.min(downsampled_scan, axis=1)
        self.last_guldenring_scan = downsampled_scan  # store to visualize later
        guldenring_obs[0, :_90, 0] = downsampled_scan
        # fill in waypoints with current goal
        for n_wpt in range(_8):
            guldenring_obs[0, _90 + n_wpt * 2:_90 + n_wpt * 2 + 2, 0] = robotstate[:2]
        # Discretize to a resolution of 5cm.
        guldenring_obs = np.round(np.divide(guldenring_obs, 0.05))*0.05
        return guldenring_obs

    def _convert_action(self, guldenring_action):
        vx, omega = guldenring_action
        ianenv_action = np.array([vx, 0., omega])
        return ianenv_action

    def step(self, guldenring_action):
        ianenv_action = self._convert_action(guldenring_action)
        ianenv_obs, reward, done, info = super(GuldenringWrapperForIANEnv, self).step(ianenv_action)
        guldenring_obs = self._convert_obs(ianenv_obs)
        return guldenring_obs, reward, done, info

    def reset(self, *args, **kwargs):
        ianenv_obs = super(GuldenringWrapperForIANEnv, self).reset(*args, **kwargs)
        guldenring_obs = self._convert_obs(ianenv_obs)
        return guldenring_obs

    def render(self, *args, **kwargs):
        lidar_angles_downsampled = np.linspace(-np.pi, np.pi, _90) \
            + self.iarlenv.rlenv.virtual_peppers[0].pos[2]
        kwargs["lidar_angles_override"] = lidar_angles_downsampled
        kwargs["lidar_scan_override"] = self.last_guldenring_scan
        return self.iarlenv.render(*args, **kwargs)


if __name__ == '__main__':
    args, _ = parse_common_args()

    if args.n is None:
        args.n = 1000
    collect_trajectories = False

    env = GuldenringWrapperForIANEnv(silent=True, collect_trajectories=collect_trajectories)
    policy = GuldenringCPolicy()

    S = run_test_episodes(env, policy, render=args.render, num_episodes=args.n)

    DIR = os.path.expanduser("~/navrep/eval/crosstest")
    if args.dry_run:
        DIR = "/tmp/navrep/eval/crosstest"
    make_dir_if_not_exists(DIR)

    if collect_trajectories:
        NAME = "guldenring_in_ianenv_{}.pckl".format(len(S))
        PATH = os.path.join(DIR, NAME)
        S.to_pickle(PATH)
    else:
        NAME = "guldenring_in_ianenv_{}.csv".format(len(S))
        PATH = os.path.join(DIR, NAME)
        S.to_csv(PATH)
    print("{} written.".format(PATH))



#!/usr/bin/env python
import gc
import numpy as np
import os
from pepper_2d_iarlenv import parse_iaenv_args, IARLEnv, check_iaenv_args
from pyniel.python_tools.timetools import WalltimeRate
from timeit import default_timer as timer

from navrep.envs.scenario_list import set_scenario, scenarios


if __name__ == "__main__":
    for scenario_name in scenarios:
        ## setting up the sim ---------------
        tic = timer()
        # args
        args = parse_iaenv_args()
        args.unmerged_scans = False
        args.continuous = True
        args.naive_plan = True
        args.no_ros = True
        set_scenario(args, scenario_name=scenario_name)
        check_iaenv_args(args)
        # env
        iarlenv = IARLEnv(args, silent=True)
        #         gc.collect()
        iarlenv.reset()
        action = np.array([0.0, 0.0, 0.0])
        toc = timer()
        setup_time = toc - tic
        tic = timer()
        N_STEPS = 1000
        for i in range(N_STEPS):
            obs, rew, done, _ = iarlenv.step(action, ONLY_FOR_AGENT_0=True)
        toc = timer()
        episode_time = toc - tic
        print(
            "Scenario {} - {} steps - setup: {:.1f}s - episode {:.1f}s".format(
                scenario_name, N_STEPS, setup_time, episode_time
            )
        )



from datetime import datetime
import os
import numpy as np

from stable_baselines import TD3
from stable_baselines.td3.policies import MlpPolicy
from stable_baselines.ddpg.noise import NormalActionNoise

from navrep.envs.encodedenv import EncodedEnv
from navrep.tools.sb_eval_callback import NavrepLogCallback
from navrep.tools.commonargs import parse_common_args

if __name__ == "__main__":
    args, _ = parse_common_args()

    DIR = os.path.expanduser("~/navrep/models/gym")
    LOGDIR = os.path.expanduser("~/navrep/logs/gym")
    START_TIME = datetime.now().strftime("%Y_%m_%d__%H_%M_%S")
    LOGNAME = "encodedenv_" + START_TIME + "_TD3"
    LOGPATH = os.path.join(LOGDIR, LOGNAME + ".csv")
    MODELPATH = os.path.join(DIR, LOGNAME + "_ckpt")
    MODELPATH2 = os.path.join(DIR, "encodedenv_latest_TD3_ckpt")
    if not os.path.exists(DIR):
        os.makedirs(DIR)
    if not os.path.exists(LOGDIR):
        os.makedirs(LOGDIR)

    env = EncodedEnv(args.backend, args.encoding, silent=True)
    cb = NavrepLogCallback(logpath=LOGPATH, savepath=MODELPATH, verbose=1)

    # The noise objects for TD3
    n_actions = env.action_space.shape[-1]
    action_noise = NormalActionNoise(mean=np.zeros(n_actions), sigma=0.1 * np.ones(n_actions))

    model = TD3(MlpPolicy, env, action_noise=action_noise, verbose=1)
    model.learn(total_timesteps=40000000, callback=cb)
    model.save(MODELPATH)
    model.save(MODELPATH2)

    del model

    model = TD3.load(MODELPATH)

    env = EncodedEnv(args.backend, args.encoding, silent=True)
    obs = env.reset()
    for i in range(512):
        action, _states = model.predict(obs, deterministic=True)
        obs, rewards, done, info = env.step(action)
        if done:
            env.reset()
        env.render()



import numpy as np
import time
from pandas import DataFrame
from stable_baselines.common.callbacks import BaseCallback
from stable_baselines.common.vec_env import VecEnv

from navrep.scripts.test_navrep import run_test_episodes, NavRepCPolicy


class NavrepLogCallback(BaseCallback):
    """
    A custom callback that derives from ``BaseCallback``.

    :param logpath: (string) where to save the training log
    :param verbose: (int) Verbosity level 0: not output 1: info 2: debug
    :param verbose: (int) Verbosity level 0: not output 1: info 2: debug
    """
    def __init__(self, logpath=None, savepath=None, eval_freq=10000, verbose=0):
        super(NavrepLogCallback, self).__init__(verbose)
        # self.model = None  # type: BaseRLModel
        # self.training_env = None  # type: Union[gym.Env, VecEnv, None]
        self.logpath = logpath
        self.savepath = savepath
        self.eval_freq = eval_freq
        self.last_len_statistics = 0
        self.best_avg_reward = [-np.inf]

    def _on_step(self) -> bool:
        """
        This method will be called by the model after each call to `env.step()`.
        :return: (bool) If the callback returns False, training is aborted early.
        """
        if self.eval_freq > 0 and self.n_calls % self.eval_freq == 0:
            # get episode_statistics
            env = self.training_env
            if isinstance(self.training_env, VecEnv):
                S = DataFrame()
                for env in self.training_env.envs:
                    S = S.append(env.episode_statistics, ignore_index=True)
                S["total_steps"] *= len(self.training_env.envs)
                S = S.sort_values(by="total_steps", ignore_index=True)
            else:
                S = env.episode_statistics

            new_S = S[self.last_len_statistics:]
            new_avg_reward = np.mean(new_S["reward"].values)

            save_log(S, self.logpath, self.verbose)
            print_statistics(new_S, 0, 0, self.n_calls, self.num_timesteps, self.verbose)
            save_model_if_improved(new_avg_reward, self.best_avg_reward, self.model, self.savepath)

            self.last_len_statistics = len(S)
        return True


class NavrepEvalCallback(BaseCallback):
    """
    A custom callback that derives from ``BaseCallback``.

    :param eval_env: (gym.Env) environment with which to evaluate the model (at eval_freq)
    :param test_env_fn: (function) function which returns an environment which is used to evaluate
                        the model after every tenth evaluation.
    :param n_eval_episodes: (int) how many episodes to run the evaluation env for
    :param logpath: (string) where to save the training log
    :param savepath: (string) where to save the model
    :param eval_freq: (int) how often to run the evaluation
    :param verbose: (int) Verbosity level 0: not output 1: info 2: debug
    :param render: (bool) human rendering in the test env
    """
    def __init__(self, eval_env, test_env_fn=None,
                 n_eval_episodes=20, logpath=None, savepath=None, eval_freq=10000, verbose=0,
                 render=False):
        super(NavrepEvalCallback, self).__init__(verbose)
        # self.model = None  # type: BaseRLModel
        # self.training_env = None  # type: Union[gym.Env, VecEnv, None]
        self.logpath = logpath
        self.savepath = savepath
        self.eval_freq = eval_freq
        self.last_len_statistics = 0
        self.best_avg_reward = [-np.inf]
        self.eval_env = eval_env
        self.test_env_fn = test_env_fn
        self.last_eval_time = time.time()
        self.render = render

    def _on_step(self) -> bool:
        """
        This method will be called by the model after each call to `env.step()`.
        :return: (bool) If the callback returns False, training is aborted early.
        """
        if self.eval_freq > 0 and self.n_calls % self.eval_freq == 0 or self.n_calls == 1:
            # get episode_statistics
            tic = time.time()
            S = run_k_episodes(self.model, self.eval_env, 20)
            toc = time.time()
            eval_duration = toc - tic

            new_avg_reward = -np.inf
            # every 10 evaluations, do a more thorough test
            if self.n_calls % (self.eval_freq * 10) == 0 or self.n_calls == 1:
                if self.test_env_fn is not None:
                    policy = NavRepCPolicy(model=self.model)
                    test_env = self.test_env_fn()
                    success_rate, avg_nav_time = run_test_episodes(test_env, policy,
                                                                   num_episodes=100, render=self.render)
                    # we add a first point to the history log, for nice plotting
                    S.loc[len(S)] = [
                        self.eval_env.total_steps,
                        'navrepval',
                        0,
                        avg_nav_time / test_env._get_dt(),
                        success_rate,
                        success_rate * 100.,
                        test_env.soadrl_sim.human_num,
                        test_env.soadrl_sim.num_walls,
                        time.time(),
                    ]
                    new_avg_reward = success_rate

            new_S = S[self.last_len_statistics:]
            if self.test_env_fn is None:
                new_avg_reward = np.mean(new_S["reward"].values)

            elapsed = time.time() - self.last_eval_time
            self.last_eval_time = time.time()

            save_log(S, self.logpath, self.verbose)
            print_statistics(new_S, elapsed, eval_duration, self.n_calls, self.num_timesteps, self.verbose)
            save_model_if_improved(new_avg_reward, self.best_avg_reward, self.model, self.savepath)

            self.last_len_statistics = len(S)
        return True

def save_log(S, logpath, verbose):
    # save log
    if logpath is not None:
        S.to_csv(logpath)
        if verbose > 1:
            print("log saved to {}.".format(logpath))

def print_statistics(S, elapsed, eval_elapsed, n_calls, n_train_steps, verbose):
    scenarios = sorted(list(set(S["scenario"].values)))
    rewards = S["reward"].values
    # print statistics
    if verbose > 0:
        print("Step {} (Env) {} (Train) - {} completed episodes - {:.0f}s (All) {:.0f}s (Eval)".format(
            n_calls, n_train_steps, len(S), elapsed, eval_elapsed))
        for scenario in scenarios:
            is_scenario = S["scenario"].values == scenario
            scenario_rewards = rewards[is_scenario]
            print("{}: {:.4f} ({})".format(
                scenario, np.mean(scenario_rewards), len(scenario_rewards)))

def save_model_if_improved(new_avg_reward, best_avg_reward, model, savepath):
    if new_avg_reward > best_avg_reward[0]:
        best_avg_reward[0] = new_avg_reward
        if savepath is not None:
            try:
                model.save(savepath)
                print("model saved to {} (avg reward: {}).".format(
                    savepath, new_avg_reward))
            except:  # noqa
                print("Could not save")

def run_k_episodes(model, env, k):
    for i in range(k):
        obs = env.reset()
        done = False
        while not done:
            action, _ = model.predict(obs, deterministic=True)
            obs, _, done, _ = env.step(action)
    return env.episode_statistics



from __future__ import print_function
import numpy as np
import os
from navrep.models.vae2d import ConvVAE, reset_graph
from navrep.tools.rings import generate_rings
from pyniel.python_tools.path_tools import make_dir_if_not_exists

# create network
reset_graph()
imvae = ConvVAE(batch_size=1, is_training=True, channels=3,)
imvae.load_json(os.path.expanduser("~/navrep/models/V/imvae.json"))

# rings converter
rings_def = generate_rings(64, 64)

# labels to learn are x, r, d (obs, reward, done)
dataset_folder = os.path.expanduser("~/navrep/datasets/V/im")


files = []
for dirpath, dirnames, filenames in os.walk(dataset_folder):
    for filename in [f for f in filenames if f.endswith(".npz")]:
        files.append(os.path.join(dirpath, filename))
files = sorted(files)
for path in files:
    arrays = np.load(path)
    images = arrays["images"]
    rewards = arrays["rewards"]
    actions = arrays["actions"]
    dones = arrays["dones"]
    obs = images.astype(float) / 255.0

    mus = []
    logvars = []
    for i in range(len(obs)):
        mu, logvar = imvae.encode_mu_logvar(obs[i:i+1, :, :, :])
        z = mu + np.exp(logvar / 2.0) * np.random.randn(*logvar.shape)
        mus.append(mu[0])
        logvars.append(logvar[0])
    mus = np.array(mus).astype(np.float32)
    logvars = np.array(logvars).astype(np.float32)
    savepath = path.replace("imvae", "imrnn").replace("images", "mus_logvars")
    make_dir_if_not_exists(os.path.dirname(savepath))
    np.savez_compressed(
        savepath,
        mus=mus,
        logvars=logvars,
        rewards=rewards,
        actions=actions,
        dones=dones,
    )
    print("z saved to {}".format(savepath))



import os

from navrep.tools.envplayer import EnvPlayer
from navrep.envs.markenv import MarkEnv

if __name__ == "__main__":
    os.environ["CUDA_VISIBLE_DEVICES"] = "-1"  # disable GPU

    env = MarkEnv()
    player = EnvPlayer(env)



from navrep.tools.envplayer import EnvPlayer
from navrep.envs.archiveenv import ArchiveEncodedEnv
from navrep.tools.commonargs import parse_common_args

if __name__ == "__main__":
    args, _ = parse_common_args()

    env = ArchiveEncodedEnv(args.backend, args.encoding, "~/navrep/datasets/V/irl")
    player = EnvPlayer(env, render_mode="rings", step_by_step=True)



# distutils: language=c++

import numpy as np
cimport numpy as np
from libc.math cimport ceil as cceil

def fast_lidar_to_rings(scans, angle_levels, range_levels, range_level_mins, range_level_maxs):
    rings = np.zeros(
        (scans.shape[0], angle_levels, range_levels, 1), dtype=np.uint8
    )
    clidar_to_rings(scans, angle_levels, range_levels, range_level_mins, range_level_maxs, rings)
    return rings

# cdef clidar_to_rings():
def clidar_to_rings(
    np.float32_t[:, ::1] scans,
    int angle_levels,
    int range_levels,
    np.float32_t[::1] range_level_mins,
    np.float32_t[::1] range_level_maxs,
    np.uint8_t[:, :, :, ::1] rings,
):
    """
    scans: ndarray (n_scans, n_rays)   0-100 [m]
    rings: ndarray (n_scans, angle_levels, range_levels, n_channels)
    """
    cdef int n_scans = scans.shape[0]
    cdef int n_rays = scans.shape[1]
    cdef int CHANNEL = 0
    cdef np.float32_t downsample_factor = n_rays * 1.0 / angle_levels  # >= 1
    # vars
    cdef int ray_idx_start
    cdef int ray_idx_end
    cdef int angle_idx
    cdef int range_idx
    cdef int scan_idx
    cdef int ray_idx
    cdef np.float32_t r_min
    cdef np.float32_t r_max
    cdef np.float32_t dist
    cdef int cell_value
    for angle_idx in range(angle_levels):
        # generate ray indices corresponding to current angle level
        ray_idx_start = int(cceil(angle_idx * downsample_factor))
        ray_idx_end = int(cceil((angle_idx + 1) * downsample_factor))
        for range_idx in range(range_levels):
            r_min = range_level_mins[range_idx]
            r_max = range_level_maxs[range_idx]
            for scan_idx in range(n_scans):
                # initialize value of cell at (angle_idx, range_idx)
                cell_value = 0
                # check if any relevant rays hit cell or fall short
                for ray_idx in range(ray_idx_start, ray_idx_end):
                    dist = scans[scan_idx, ray_idx]
                    if dist == 0:
                        continue
                    # if a ray hits the cell the value is 2
                    if r_min <= dist and dist < r_max:
                        cell_value = 2
                        break
                    # if a ray falls short of the cell the value is at least 1
                    if dist < r_min:
                        cell_value = 1
                        continue
                rings[scan_idx, angle_idx, range_idx, CHANNEL] = cell_value



import numpy as np
import os
import configparser
import tensorflow as tf
from pkg_resources import resource_filename
from pyniel.python_tools.path_tools import make_dir_if_not_exists

import crowd_sim  # adds CrowdSim-v0 to gym  # noqa
from crowd_sim.envs.crowd_sim import CrowdSim  # reference to env code  # noqa
from crowd_sim.envs.utils.robot import Robot  # next line fails otherwise # noqa
from crowd_nav.policy.network_om import SDOADRL
from crowd_sim.envs.utils.state import JointState, FullState, ObservableState
from crowd_sim.envs.utils.action import ActionRot

from navrep.scripts.cross_test_navreptrain_in_ianenv import run_test_episodes
from navrep.tools.commonargs import parse_common_args
from navrep.envs.ianenv import IANEnv

TODO = None

class LuciaRawPolicy(object):
    """ legacy SOADRL policy from lucia's paper, takes in agents state, local map
    The problem is that in the original implementation, policy and environment are intertwined.
    this class goes further into separating them by reimplementing methods from
    agents.py, robots.py """

    def __init__(self):
        self._make_policy()

    def _make_policy(self):
        # Config
        config_dir = resource_filename('crowd_nav', 'config')
        config_file = os.path.join(config_dir, 'test_soadrl_static.config')
        config = configparser.RawConfigParser()
        config.read(config_file)

        sess = tf.Session()
        policy = SDOADRL()
        policy.configure(sess, 'global', config)
        policy.set_phase('test')
        self.model_path = os.path.expanduser('~/soadrl/Final_models/angular_map_full_FOV/rl_model')
        policy.load_model(self.model_path)

        self.policy = policy

    def act(self, obs):
        robot_state, humans_state, local_map = obs
        state = JointState(robot_state, humans_state)
        action = self.policy.predict(state, local_map, None)
        action = ActionRot(robot_state.v_pref * action.v, action.r)  # de-normalize
        return action

class IANEnvWithLegacySOADRLObs(object):
    def __init__(self,
                 silent=False, max_episode_length=1000, collect_trajectories=False):
        # Get lidar values from the SOADRL config
        config_dir = resource_filename('crowd_nav', 'config')
        config_file = os.path.join(config_dir, 'test_soadrl_static.config')
        config = configparser.RawConfigParser()
        config.read(config_file)
        self.v_pref = config.getfloat('humans', 'v_pref')
        # lidar scan expected by SOADRL
        self.angular_map_max_range = config.getfloat('map', 'angular_map_max_range')
        self.angular_map_dim = config.getint('map', 'angular_map_dim')
        self.angular_map_min_angle = config.getfloat('map', 'angle_min') * np.pi
        self.angular_map_max_angle = config.getfloat('map', 'angle_max') * np.pi
        self.angular_map_angle_increment = (
            self.angular_map_max_angle - self.angular_map_min_angle) / self.angular_map_dim
        self.lidar_upsampling = 15

        # create env
        self.env = IANEnv(
            silent=silent, max_episode_length=max_episode_length, collect_trajectories=collect_trajectories)
        self.reset()

    def reset(self):
        """ IANEnv destroys and re-creates its iarlenv at every reset, so apply our changes here """
        self.env.reset()

        # we raytrace at a higher resolution, then downsample back to the original soadrl resolution
        # this avoids missing small obstacles due to the small soadrl resolution
        self.env.iarlenv.rlenv.virtual_peppers[0].kLidarMergedMaxAngle = self.angular_map_max_angle
        self.env.iarlenv.rlenv.virtual_peppers[0].kLidarMergedMinAngle = self.angular_map_min_angle
        self.env.iarlenv.rlenv.virtual_peppers[0].kLidarAngleIncrement = \
            self.angular_map_angle_increment / self.lidar_upsampling
        self.env.iarlenv.rlenv.kMergedScanSize = self.angular_map_dim * self.lidar_upsampling

        self.episode_statistics = self.env.episode_statistics

        obs, _, _, _ = self.step(ActionRot(0.,0.))
        return obs

    def step(self, action):
        # convert lucia action to IANEnv action
        ianenv_action = np.array([0., 0., 0.])
        # SOADRL - rotation is dtheta
        # IAN    - rotation is dtheta/dt
        ianenv_action[2] = action.r / self.env._get_dt()
        #  SOADRL - instant rot, then vel
        #  IAN    - vel, then rot
        action_vy = 0. # SOADRL outputs non-holonomic by default
        ianenv_action[0] = action.v * np.cos(action.r) - action_vy * np.sin(action.r)
        ianenv_action[1] = action.v * np.sin(action.r) + action_vy * np.cos(action.r)
        # get obs from IANEnv
        obs, rew, done, info = self.env.step(ianenv_action)
        # convert to SOADRL style
        robot_state = FullState(
            self.env.iarlenv.rlenv.virtual_peppers[0].pos[0],
            self.env.iarlenv.rlenv.virtual_peppers[0].pos[1],
            self.env.iarlenv.rlenv.virtual_peppers[0].vel[0],
            self.env.iarlenv.rlenv.virtual_peppers[0].vel[1],
            self.env.iarlenv.rlenv.vp_radii[0],
            self.env.iarlenv.rlenv.agent_goals[0][0],
            self.env.iarlenv.rlenv.agent_goals[0][1],
            self.v_pref,
            self.env.iarlenv.rlenv.virtual_peppers[0].pos[2],)
        humans_state = [ObservableState(
            human.pos[0],
            human.pos[1],
            human.vel[0],
            human.vel[1],
            r,) for human, r in zip(
                self.env.iarlenv.rlenv.virtual_peppers[1:], self.env.iarlenv.rlenv.vp_radii[1:])]
        scan = obs[0]
        # for each angular section we take the min of the returns
        downsampled_scan = scan.reshape((-1, self.lidar_upsampling))
        downsampled_scan = np.min(downsampled_scan, axis=1)
        self.last_downsampled_scan = downsampled_scan
        local_map = np.clip(downsampled_scan / self.angular_map_max_range, 0., 1.)
        obs = (robot_state, humans_state, local_map)
        return obs, rew, done, info

    def _get_dt(self):
        return self.env._get_dt()

    def render(self, *args, **kwargs):
        _, lidar_angles = self.env.iarlenv.rlenv.virtual_peppers[0].get_lidar_update_ijangles(
            "merged", self.env.iarlenv.rlenv.kMergedScanSize
        )
        lidar_angles_downsampled = lidar_angles[::self.lidar_upsampling]
        kwargs["lidar_angles_override"] = lidar_angles_downsampled
        kwargs["lidar_scan_override"] = self.last_downsampled_scan
        return self.env.render(*args, **kwargs)


if __name__ == '__main__':
    args, _ = parse_common_args()

    if args.n is None:
        args.n = 1000
    collect_trajectories = False

    env = IANEnvWithLegacySOADRLObs(silent=True, collect_trajectories=collect_trajectories)
    policy = LuciaRawPolicy()

    S = run_test_episodes(env, policy, render=args.render, num_episodes=args.n)

    DIR = os.path.expanduser("~/navrep/eval/crosstest")
    if args.dry_run:
        DIR = "/tmp/navrep/eval/crosstest"
    make_dir_if_not_exists(DIR)
    if collect_trajectories:
        NAME = "lucianavreptrain_in_ianenv_{}.pckl".format(len(S))
        PATH = os.path.join(DIR, NAME)
        S.to_pickle(PATH)
    else:
        NAME = "lucianavreptrain_in_ianenv_{}.csv".format(len(S))
        PATH = os.path.join(DIR, NAME)
        S.to_csv(PATH)
    print("{} written.".format(PATH))



import os
import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
import matplotlib

from navrep.scripts.plot_gym_training_progress import get_backend_and_encoding, get_envname, get_date, \
    get_color_and_style, color_and_style, get_label
from navrep.tools.commonargs import parse_plotting_args

matplotlib.rcParams['mathtext.fontset'] = 'stix'
matplotlib.rcParams['font.family'] = 'STIXGeneral'
matplotlib.rcParams['font.size'] = 17

ALL_RUNS = False

backend_ranks = {
    "VAE1D_LSTM": 0,
    "VAE_LSTM": 3,
    "VAE1DLSTM": 1,
    "VAELSTM": 4,
    "GPT1D": 2,
    "GPT": 5,
    None: 0,
}
envname_ranks = {
    "e2e1dnavreptrain": 0,
    "e2enavreptrain": 1,
    "lucianavreptrain": 2,
    "navreptrainencodedenv": 3,
}
encoding_ranks = {
    "V_ONLY": 0,
    "M_ONLY": 1,
    "VM": 2,
    None: 0,
}
def sort_by_env_backend_encoding_date(logs):
    sortkeys = {}
    for log in logs:
        backend, encoding = get_backend_and_encoding(log)
        envname = get_envname(log)
        date = get_date(log)
        try:
            sortkeys[log] = (envname_ranks[envname], backend_ranks[backend], encoding_ranks[encoding], date)
        except KeyError as e:
            print(e)
            sortkeys[log] = (0, 0, 0, '')

    sorted_logs = sorted(logs, key=lambda x:sortkeys[x])
    return sorted_logs


def style_to_hatch(style):
    hatch = None
    if style == "dotted":
        hatch = "\\"
    if style == "dashed":
        hatch = "//"
    if style == "solid":
        hatch = None
    return hatch

def calculate_bar_offsets(bar_width, n_bars):
    offsets = np.linspace(bar_width / 2. - 0.8 / 2., 0.8 / 2 - bar_width / 2, n_bars)
    return offsets


if __name__ == "__main__":
    args, _ = parse_plotting_args()

    LOGDIR = args.logdir
    if LOGDIR is None:
        LOGDIR = "~/navrep"
    LOGDIR = os.path.join(LOGDIR, "eval/crosstest")
    LOGDIR = os.path.expanduser(LOGDIR)
#     logs = [log for log in os.listdir(LOGDIR) if log.endswith(".pckl")]
    logs = [log for log in os.listdir(LOGDIR) if log.endswith(".csv")]
    logs = sort_by_env_backend_encoding_date(logs)
    Ss = []
    log_scenarios = []
    log_success_rates = []
    for log in logs:
        path = os.path.join(LOGDIR, log)
        S = pd.read_csv(path)
#         S = pd.read_pickle(path)

        print()
        print()
        print(" ----------------------------------")
        print(" | {:^30.30} |".format(log))
        print(" ----------------------------------")
        print()

        all_success_rate = np.mean(S["goal_reached"])
        all_avg_reward = np.mean(S["reward"])
        print("ALL: {:.1f}% - R={:.1f}".format(100.*all_success_rate, all_avg_reward))
        print("-----------")

        scenarios = np.array(sorted(list(set(S["scenario"]))))
        success_rates = []
        for scenario in scenarios:
            scenario_specific = S[S["scenario"] == scenario]
            success_rate = np.mean(scenario_specific["goal_reached"])
            avg_reward = np.mean(scenario_specific["reward"])
            n = len(scenario_specific["reward"])
            print("{} ({}): {:.1f}%".format(scenario, n, 100.*success_rate))
            success_rates.append(success_rate)

        Ss.append(S)
        log_scenarios.append(scenarios)
        log_success_rates.append(success_rates)

    all_scenarios = sorted(list(set([s for l_ in log_scenarios for s in l_])))
    scenario_idx = {s: i for i, s in enumerate(all_scenarios)}

    if ALL_RUNS:
        # Plot per scenario

        fig, ax = plt.subplots()
        width = 0.8 / len(log_scenarios)
        offsets = calculate_bar_offsets(width, len(log_scenarios))

        for legend, x_labels, y_values, o in zip(logs, log_scenarios, log_success_rates, offsets):
            x = np.array([scenario_idx[s] for s in x_labels])
            color, style = get_color_and_style(legend)
            hatch = style_to_hatch(style)
            plt.bar(x + o, y_values, width, color=color, edgecolor="white", hatch=hatch, label=legend)

        xticks = range(len(all_scenarios))
        plt.xticks(xticks, all_scenarios, rotation='vertical')
        plt.legend(bbox_to_anchor=(1.05, 1.))

    if ALL_RUNS:
        # Group per map
        all_groups = ["asl", "unity_scene_map", "asl_office_j"]
        group_success_rates = []
        for S in Ss:
            success_rates = []
            for group in all_groups:
                if group == "asl":
                    scenario_specific = S[~S["scenario"].str.contains("asl_office_j")
                                          & ~S["scenario"].str.contains("unity_scene_map")]
                else:
                    scenario_specific = S[S["scenario"].str.contains(group)]
                success_rate = np.mean(scenario_specific["goal_reached"])
                success_rates.append(success_rate)
            group_success_rates.append(success_rates)

        fig, ax = plt.subplots()
        for legend, y_values, o in zip(logs, group_success_rates, offsets):
            x_labels = all_groups
            x = np.arange(len(x_labels))
            color, style = get_color_and_style(legend)
            hatch = style_to_hatch(style)
            plt.bar(x + o, y_values, width, color=color, edgecolor="white", hatch=hatch, label=legend)
        xticks = range(len(all_groups))
        plt.xticks(xticks, all_groups, rotation='vertical')
        plt.legend(bbox_to_anchor=(1.05, 1.))

    if ALL_RUNS:
        # Total
        total_success_rates = []
        for S in Ss:
            success_rate = np.mean(S["goal_reached"])
            total_success_rates.append([success_rate])

        fig, ax = plt.subplots()
        for legend, y_values, o in zip(logs, total_success_rates, offsets):
            x_labels = ["all scenarios"]
            x = np.arange(len(x_labels))
            color, style = get_color_and_style(legend)
            hatch = style_to_hatch(style)
            plt.bar(x + o, y_values, width, color=color, edgecolor='white', hatch=hatch, label=legend)
        xticks = range(len(x_labels))
        plt.xticks(xticks, x_labels, rotation='vertical')
        plt.legend(bbox_to_anchor=(1.05, 1.))

    # ----------------------------------------------------------
    # Paper

    # calculate specie success rates
    # group per encoding
    species_success_rates = {}
    map_groups = ["all", "asl", "unity_scene_map", "asl_office_j"]
    scenario_groups = all_scenarios
    all_groups = map_groups + scenario_groups
    for log, S in zip(logs, Ss):
        backend, encoding = get_backend_and_encoding(log)
        envname = get_envname(log)
        specie_key = (envname, backend, encoding)
        if specie_key not in species_success_rates:
            species_success_rates[specie_key] = {group: [] for group in all_groups}

        # get per-map success rate
        for group in map_groups:
            if group == "asl":
                group_specific = S[~S["scenario"].str.contains("asl_office_j")
                                   & ~S["scenario"].str.contains("unity_scene_map")]
            elif group == "all":
                group_specific = S
            else:
                group_specific = S[S["scenario"].str.contains(group)]
            success_rate = np.mean(group_specific["goal_reached"])

            species_success_rates[specie_key][group].append(success_rate)

        # get per-scenario success rate
        for group in scenario_groups:
            group_specific = S[S["scenario"] == group]
            success_rate = np.mean(group_specific["goal_reached"])

            species_success_rates[specie_key][group].append(success_rate)

    # table
    table_groups = [
        "irosasl2",
        "irosasl6",
        "irosasl3",
        "irosasl1",
        "irosasl4",
        "irosasl5",
        "rlasl1",
        "rlasl2",
        "rlasl3",
        "irosunity_scene_map2",
        "irosunity_scene_map6",
        "irosunity_scene_map3",
        "irosunity_scene_map1",
        "irosunity_scene_map4",
        "irosunity_scene_map5",
        "rlunity_scene_map1",
        "rlunity_scene_map2",
        "rlunity_scene_map3",
        "irosasl_office_j2",
        "irosasl_office_j6",
        "irosasl_office_j3",
        "irosasl_office_j1",
        "irosasl_office_j4",
        "irosasl_office_j5",
        "rlasl_office_j1",
        "rlasl_office_j2",
        "rlasl_office_j3",
    ]
    group_labels = {
        "irosasl2": "1",
        "irosasl6": "2",
        "irosasl3": "3",
        "irosasl1": "4",
        "irosasl4": "5",
        "irosasl5": "6",
        "rlasl1": "7",
        "rlasl2": "8",
        "rlasl3": "9",
        "irosunity_scene_map2": "1",
        "irosunity_scene_map6": "2",
        "irosunity_scene_map3": "3",
        "irosunity_scene_map1": "4",
        "irosunity_scene_map4": "5",
        "irosunity_scene_map5": "6",
        "rlunity_scene_map1": "7",
        "rlunity_scene_map2": "8",
        "rlunity_scene_map3": "9",
        "irosasl_office_j2": "1",
        "irosasl_office_j6": "2",
        "irosasl_office_j3": "3",
        "irosasl_office_j1": "4",
        "irosasl_office_j4": "5",
        "irosasl_office_j5": "6",
        "rlasl_office_j1": "7",
        "rlasl_office_j2": "8",
        "rlasl_office_j3": "9",
    }
    group_best = {group: -np.inf for group in table_groups}
    for specie_key in species_success_rates:
        specie_success_rates = species_success_rates[specie_key]
        for group in table_groups:
            success = np.max(specie_success_rates[group]) * 100.
            if success > group_best[group]:
                group_best[group] = success
    print()
    print("Success rates LaTEX table:")
    print("--------------------------")
    c_s = " ".join(["c"] * (len(table_groups) + 1))
    print('\\begin{tabular}{ ' + "{}".format(c_s) + "}")
    print('\\toprule')
    print('& \\multicolumn{9}{c}{simple} & \\multicolumn{9}{|c|}{complex} & \\multicolumn{9}{c}{realistic} \\' '\\')  # noqa
    for group in table_groups:
        group_label = group_labels[group]
        print("& {}    ".format(group_label), end="")
    print('\\'+'\\')
    print('\\midrule')
    for specie_key in species_success_rates:
        envname, backend, encoding = specie_key
        specie_label = get_label(envname, backend, encoding)
        specie_success_rates = species_success_rates[specie_key]

        print("{:<10} ".format(specie_label), end="")
        for group in table_groups:
            n_trials = len(specie_success_rates[group])
            planner_scene_avgsr = "--"
            if n_trials != 0:
                success = np.max(specie_success_rates[group]) * 100.
                planner_scene_avgsr = "{:.1f}".format(success)
                if success == group_best[group] and success != 0:
                    planner_scene_avgsr = '\\textbf{' + planner_scene_avgsr + '}'
            print("& {:>10} ".format(planner_scene_avgsr), end="")
        print('\\'+'\\')
    print('\\bottomrule')
    print('\\end{tabular}')
    print()

    # box plot
    width = 0.8 / len(species_success_rates)
    offsets = calculate_bar_offsets(width, len(species_success_rates))

    fig, ax = plt.subplots()
    for key, o in zip(species_success_rates, offsets):
        envname, backend, encoding = key
        label = get_label(envname, backend, encoding)
        specie_success_rates = species_success_rates[key]
        ymin = np.array([np.min(specie_success_rates[group]) for group in map_groups])
        ymax = np.array([np.max(specie_success_rates[group]) for group in map_groups])
        ymean = np.array([np.mean(specie_success_rates[group]) for group in map_groups])
        yn = np.array([len(specie_success_rates[group]) for group in map_groups])
        x_labels = map_groups
        x = np.arange(len(x_labels))
        color, style = color_and_style(encoding, backend, envname)
        hatch = style_to_hatch(style)
        BOX_PLOT = False
        if BOX_PLOT:
            ymin = ymin - 0.005
            ymax = ymax + 0.005
            plt.bar(x + o, ymax-ymin, width, bottom=ymin, # box plot
                    color=color, edgecolor="white", hatch=hatch, label=label)
            for xi, ymeani in zip(x, ymean):
                plt.plot([xi + o - width/2., xi + o + width/2.], [ymeani, ymeani], color='dimgray')
        else:
            plt.bar(x + o, ymean, width,
                    color=color, edgecolor="white", hatch=hatch, label=label)
            # error bars
            for xi, ymini, ymaxi, yni in zip(x, ymin, ymax, yn):
                if yni > 1:
                    plt.plot([xi + o, xi + o], [ymini, ymaxi], color='dimgray')
    xticks = np.arange(len(map_groups))
    vlines = xticks[:-1] + 0.5
    for vline in vlines:
        plt.axvline(vline, color='dimgray', linewidth=1)
    plt.xticks(xticks, map_groups, rotation='vertical')
    plt.legend(bbox_to_anchor=(1.05, 1.))

    plt.show()



import os

from navrep.envs.dreamenv import DreamEnv

class ToyDreamEnv(DreamEnv):
    def __init__(self, temperature=0.25):
        super(ToyDreamEnv, self).__init__(
            temperature=temperature,
            initial_z_path=os.path.expanduser(
                "~/navrep/datasets/M/toy/000_mus_logvars_robotstates_actions_rewards_dones.npz"
            ),
            rnn_model_path=os.path.expanduser("~/navrep/models/M/toyrnn.json"),
            vae_model_path=os.path.expanduser("~/navrep/models/V/toyvae.json"),
        )



from __future__ import print_function
import numpy as np
import os
from sensor_msgs.msg import LaserScan

from navrep.tools.data_extraction import archive_to_lidar_dataset
from navrep.models.vae1d import Conv1DVAE, reset_graph

DEBUG_PLOTTING = True

# Parameters for training
batch_size = 100
N_SCANS_PER_BATCH = 1
NUM_EPOCH = 100
DATA_DIR = "record"
HOME = os.path.expanduser("~")
MAX_LIDAR_DIST = 25.0

vae_model_path = os.path.expanduser("~/navrep/models/V/vae1d.json")

# create network
reset_graph()
vae = Conv1DVAE(batch_size=batch_size, is_training=False)

# load
vae.load_json(vae_model_path)

# create training dataset
dataset = archive_to_lidar_dataset("~/navrep/datasets/V/ian", limit=180)
if len(dataset) == 0:
    raise ValueError("no scans found, exiting")
print(len(dataset), "scans in dataset.")
dataset = dataset[100000:]

# split into batches:
total_length = len(dataset)
num_batches = len(dataset)


dummy_msg = LaserScan()
dummy_msg.range_max = 100.0
dummy_msg.ranges = range(1080)

batch = dataset[:N_SCANS_PER_BATCH]
scans = batch

obs = np.clip(scans.astype(np.float) / MAX_LIDAR_DIST, 0.0, MAX_LIDAR_DIST)
obs = obs.reshape(N_SCANS_PER_BATCH, 1080, 1)

obs_pred = vae.encode_decode(obs)

from navrep.models.siren1d import SIREN1D, reset_graph

DEBUG_PLOTTING = True

# Parameters for training
batch_size = 1080
N_SCANS_PER_BATCH = 1
NUM_EPOCH = 100
DATA_DIR = "record"
HOME = os.path.expanduser("~")
MAX_LIDAR_DIST = 25.0

siren_model_path = os.path.expanduser("~/navrep/models/V/siren1d.json")

# create network
reset_graph()
siren = SIREN1D(batch_size=batch_size, is_training=False)

# load
siren.load_json(siren_model_path)

# split into batches:
total_length = len(dataset)
num_batches = len(dataset)

dummy_msg = LaserScan()
dummy_msg.range_max = 100.0
dummy_msg.ranges = range(1080)

batch = dataset[:N_SCANS_PER_BATCH]
scans = batch

obs = np.clip(scans.astype(np.float) / MAX_LIDAR_DIST, 0.0, MAX_LIDAR_DIST)
obs = obs.reshape(N_SCANS_PER_BATCH, 1080, 1)

obs_pred2 = siren.encode_decode(obs)

import matplotlib.pyplot as plt

plt.ion()
plt.figure("rings")
plt.cla()
plt.plot(obs[0,:,0])
plt.plot(obs_pred[0,:,0])
plt.plot(obs_pred2[0,:,0])
plt.legend(["true", "convolutional", "SIREN"])
# update
plt.pause(0.01)



import argparse
import os

def populate_common_args(parser):
    parser.add_argument('--no-gpu', action='store_true', help='disables gpu for tensorflow')
    parser.add_argument('--debug', action='store_true', help='uses slower but easier to debug methods')
    parser.add_argument('--dry-run', action='store_true', help='saves files to tmp instead of navrep folder')
    parser.add_argument('--render', action='store_true', help='forces render')
    parser.add_argument('--environment', '--env', type=str, default=None,
                        choices=['ian', 'toy', 'markone', 'marktwo', 'navreptrain', 'lucia', 'irl'],
                        help="Switches between various environments.")
    parser.add_argument('--backend', type=str, default="GPT",
                        choices=['GPT', 'GPT1D', 'VAELSTM', 'VAE1DLSTM', 'VAE_LSTM', 'VAE1D_LSTM',
                                 'E2E', 'E2E1D'],
                        help="Picks which world model architecture to use")
    parser.add_argument('--encoding', type=str, default="V_ONLY",
                        choices=['V_ONLY', 'VM', 'M_ONLY',
                                 'VCARCH', 'CARCH'],
                        help="Picks which outputs from the world models to use")
    parser.add_argument('--n', type=int, help="for training, n_steps, for generation, n_sequences, etc..")

def populate_multiproc_args(parser):
    parser.add_argument('--subprocess', nargs=2, type=int,
                        help="""[i N] i is the id of this subprocess,
                                     N is the total amount of subprocesses to split into""")

def populate_plotting_args(parser):
    parser.add_argument('--scenario', type=str)
    parser.add_argument('--logdir', type=str)
    parser.add_argument('--x-axis', type=str)
    parser.add_argument('--y-axis', type=str)

def populate_rosnode_args(parser):
    parser.add_argument('--no-stop', action='store_true', help="give planner immediate control")
    parser.add_argument('--hz', action='store_true', help="print time information")
    # TODO: do this the proper way (roslaunch remaps)
    parser.add_argument('--ian-topics', action='store_true', help="remaps topic for use with IAN")

def check_common_args(parsed_args):
    if parsed_args.no_gpu:
        os.environ["CUDA_VISIBLE_DEVICES"] = "-1"  # disable GPU

def check_multiproc_args(parsed_args):
    parsed_args.subproc_id = 0
    parsed_args.n_subprocs = 1
    if parsed_args.subprocess is not None:
        parsed_args.subproc_id = parsed_args.subprocess[0]
        parsed_args.n_subprocs = parsed_args.subprocess[1]
        if parsed_args.subproc_id >= parsed_args.n_subprocs:
            raise ValueError("subproc_id ({}) must be strictly smaller than n_subprocs ({})".format(
                parsed_args.subproc_id, parsed_args.n_subprocs))

def check_plotting_args(parsed_args):
    pass

def check_rosnode_args(parsed_args):
    pass

def parse_common_args(args=None, ignore_unknown=False):
    arg_populate_funcs = [populate_common_args]
    arg_check_funcs = [check_common_args]
    return parse_various_args(args, arg_populate_funcs, arg_check_funcs, ignore_unknown)

def parse_multiproc_args(args=None, ignore_unknown=False):
    arg_populate_funcs = [populate_common_args, populate_multiproc_args]
    arg_check_funcs = [check_common_args, check_multiproc_args]
    return parse_various_args(args, arg_populate_funcs, arg_check_funcs, ignore_unknown)

def parse_plotting_args(args=None, ignore_unknown=False):
    arg_populate_funcs = [populate_plotting_args]
    arg_check_funcs = [check_plotting_args]
    return parse_various_args(args, arg_populate_funcs, arg_check_funcs, ignore_unknown)

def parse_rosnode_args(args=None, ignore_unknown=False):
    arg_populate_funcs = [populate_rosnode_args]
    arg_check_funcs = [check_rosnode_args]
    return parse_various_args(args, arg_populate_funcs, arg_check_funcs, ignore_unknown)

def parse_various_args(args, arg_populate_funcs, arg_check_funcs, ignore_unknown):
    """ generic arg parsing function """
    parser = argparse.ArgumentParser()

    for func in arg_populate_funcs:
        func(parser)

    if ignore_unknown:
        parsed_args, unknown_args = parser.parse_known_args(args=args)
    else:
        parsed_args = parser.parse_args(args=args)
        unknown_args = []

    for func in arg_check_funcs:
        func(parsed_args)

    print_args(parsed_args)

    return parsed_args, unknown_args

def print_args(args):
    print()
    print("-------------------------------")
    print("             ARGS              ")
    for k in args.__dict__:
        print("-- {} : {}".format(k, args.__dict__[k]))
    print("-------------------------------")
    print()


if __name__ == "__main__":
    args, unknown_args = parse_common_args()



from navrep.envs.markencodedenv import MarkOneEncodedEnv
from navrep.envs.markenv import FIRST_TRAIN_MAPS, FIRST_TEST_MAPS
from navrep.scripts.test_mark_common import MarkOneCPolicy, markeval
from navrep.tools.commonargs import parse_multiproc_args


if __name__ == "__main__":
    args, _ = parse_multiproc_args()

    policy = MarkOneCPolicy()

    env = MarkOneEncodedEnv(silent=True, maps=FIRST_TRAIN_MAPS)
    S1 = markeval(policy, env,
                  n_episodes=args.n,
                  subset_index=args.subproc_id, n_subsets=args.n_subprocs,
                  render=args.render)

    env = MarkOneEncodedEnv(silent=True, maps=FIRST_TEST_MAPS)
    S2 = markeval(policy, env,
                  n_episodes=args.n,
                  subset_index=args.subproc_id, n_subsets=args.n_subprocs,
                  render=args.render)

    import matplotlib.pyplot as plt
    plt.ioff()
    plt.show()



# This Python file uses the following encoding: utf-8
"""autogenerated by genpy from frame_msgs/PersonTrajectories.msg. Do not edit."""
import sys
python3 = True if sys.hexversion > 0x03000000 else False
import genpy
import struct

import frame_msgs.msg
import geometry_msgs.msg
import genpy
import std_msgs.msg

class PersonTrajectories(genpy.Message):
  _md5sum = "ae6789945f2030885896ef815b8cdbee"
  _type = "frame_msgs/PersonTrajectories"
  _has_header = True #flag to mark the presence of a Header object
  _full_text = """# Message with all currently tracked person trajectories
#

Header              header          # Header containing timestamp etc. of this message
PersonTrajectory[]  trajectories    # All states of the last T frames of the most likely trajectory of that tracked person.

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.sec: seconds (stamp_secs) since epoch (in Python the variable is called 'secs')
# * stamp.nsec: nanoseconds since stamp_secs (in Python the variable is called 'nsecs')
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
string frame_id

================================================================================
MSG: frame_msgs/PersonTrajectory
# Message defining the trajectory of a tracked person.
#
# The distinction between track and trajectory is that, depending on the
# implementation of the tracker, a single track (i.e. tracked person) might
# change the trajectory if at some point a new trajectory "fits" that track (person)
# better.
#

uint64                   track_id   # Unique identifier of the tracked person.
PersonTrajectoryEntry[]  trajectory # All states of the last T frames of the most likely trajectory of that tracked person.

================================================================================
MSG: frame_msgs/PersonTrajectoryEntry
# Message defining an entry of a person trajectory.
#

duration    age             # age of the track
bool        is_occluded     # if the track is currently not matched by a detection
uint64      detection_id    # id of the corresponding detection in the current cycle (undefined if occluded)

# The following fields are extracted from the Kalman state x and its covariance C

geometry_msgs/PoseWithCovariance    pose   # pose of the track (z value and orientation might not be set, check if corresponding variance on diagonal is > 99999)
geometry_msgs/TwistWithCovariance   twist  # velocity of the track (z value and rotational velocities might not be set, check if corresponding variance on diagonal is > 99999)

================================================================================
MSG: geometry_msgs/PoseWithCovariance
# This represents a pose in free space with uncertainty.

Pose pose

# Row-major representation of the 6x6 covariance matrix
# The orientation parameters use a fixed-axis representation.
# In order, the parameters are:
# (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
float64[36] covariance

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of position and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: geometry_msgs/TwistWithCovariance
# This expresses velocity in free space with uncertainty.

Twist twist

# Row-major representation of the 6x6 covariance matrix
# The orientation parameters use a fixed-axis representation.
# In order, the parameters are:
# (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
float64[36] covariance

================================================================================
MSG: geometry_msgs/Twist
# This expresses velocity in free space broken into its linear and angular parts.
Vector3  linear
Vector3  angular

================================================================================
MSG: geometry_msgs/Vector3
# This represents a vector in free space. 
# It is only meant to represent a direction. Therefore, it does not
# make sense to apply a translation to it (e.g., when applying a 
# generic rigid transformation to a Vector3, tf2 will only apply the
# rotation). If you want your data to be translatable too, use the
# geometry_msgs/Point message instead.

float64 x
float64 y
float64 z"""
  __slots__ = ['header','trajectories']
  _slot_types = ['std_msgs/Header','frame_msgs/PersonTrajectory[]']

  def __init__(self, *args, **kwds):
    """
    Constructor. Any message fields that are implicitly/explicitly
    set to None will be assigned a default value. The recommend
    use is keyword arguments as this is more robust to future message
    changes.  You cannot mix in-order arguments and keyword arguments.

    The available fields are:
       header,trajectories

    :param args: complete set of field values, in .msg order
    :param kwds: use keyword arguments corresponding to message field names
    to set specific fields.
    """
    if args or kwds:
      super(PersonTrajectories, self).__init__(*args, **kwds)
      #message fields cannot be None, assign default values for those that are
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.trajectories is None:
        self.trajectories = []
    else:
      self.header = std_msgs.msg.Header()
      self.trajectories = []

  def _get_types(self):
    """
    internal API method
    """
    return self._slot_types

  def serialize(self, buff):
    """
    serialize message into buffer
    :param buff: buffer, ``StringIO``
    """
    try:
      _x = self
      buff.write(_get_struct_3I().pack(_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs))
      _x = self.header.frame_id
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      length = len(self.trajectories)
      buff.write(_struct_I.pack(length))
      for val1 in self.trajectories:
        buff.write(_get_struct_Q().pack(val1.track_id))
        length = len(val1.trajectory)
        buff.write(_struct_I.pack(length))
        for val2 in val1.trajectory:
          _v1 = val2.age
          _x = _v1
          buff.write(_get_struct_2i().pack(_x.secs, _x.nsecs))
          _x = val2
          buff.write(_get_struct_BQ().pack(_x.is_occluded, _x.detection_id))
          _v2 = val2.pose
          _v3 = _v2.pose
          _v4 = _v3.position
          _x = _v4
          buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
          _v5 = _v3.orientation
          _x = _v5
          buff.write(_get_struct_4d().pack(_x.x, _x.y, _x.z, _x.w))
          buff.write(_get_struct_36d().pack(*_v2.covariance))
          _v6 = val2.twist
          _v7 = _v6.twist
          _v8 = _v7.linear
          _x = _v8
          buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
          _v9 = _v7.angular
          _x = _v9
          buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
          buff.write(_get_struct_36d().pack(*_v6.covariance))
    except struct.error as se: self._check_types(struct.error("%s: '%s' when writing '%s'" % (type(se), str(se), str(locals().get('_x', self)))))
    except TypeError as te: self._check_types(ValueError("%s: '%s' when writing '%s'" % (type(te), str(te), str(locals().get('_x', self)))))

  def deserialize(self, str):
    """
    unpack serialized message in str into this message instance
    :param str: byte array of serialized message, ``str``
    """
    try:
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.trajectories is None:
        self.trajectories = None
      end = 0
      _x = self
      start = end
      end += 12
      (_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs,) = _get_struct_3I().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.header.frame_id = str[start:end].decode('utf-8')
      else:
        self.header.frame_id = str[start:end]
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      self.trajectories = []
      for i in range(0, length):
        val1 = frame_msgs.msg.PersonTrajectory()
        start = end
        end += 8
        (val1.track_id,) = _get_struct_Q().unpack(str[start:end])
        start = end
        end += 4
        (length,) = _struct_I.unpack(str[start:end])
        val1.trajectory = []
        for i in range(0, length):
          val2 = frame_msgs.msg.PersonTrajectoryEntry()
          _v10 = val2.age
          _x = _v10
          start = end
          end += 8
          (_x.secs, _x.nsecs,) = _get_struct_2i().unpack(str[start:end])
          _x = val2
          start = end
          end += 9
          (_x.is_occluded, _x.detection_id,) = _get_struct_BQ().unpack(str[start:end])
          val2.is_occluded = bool(val2.is_occluded)
          _v11 = val2.pose
          _v12 = _v11.pose
          _v13 = _v12.position
          _x = _v13
          start = end
          end += 24
          (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
          _v14 = _v12.orientation
          _x = _v14
          start = end
          end += 32
          (_x.x, _x.y, _x.z, _x.w,) = _get_struct_4d().unpack(str[start:end])
          start = end
          end += 288
          _v11.covariance = _get_struct_36d().unpack(str[start:end])
          _v15 = val2.twist
          _v16 = _v15.twist
          _v17 = _v16.linear
          _x = _v17
          start = end
          end += 24
          (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
          _v18 = _v16.angular
          _x = _v18
          start = end
          end += 24
          (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
          start = end
          end += 288
          _v15.covariance = _get_struct_36d().unpack(str[start:end])
          val1.trajectory.append(val2)
        self.trajectories.append(val1)
      return self
    except struct.error as e:
      raise genpy.DeserializationError(e) #most likely buffer underfill


  def serialize_numpy(self, buff, numpy):
    """
    serialize message with numpy array types into buffer
    :param buff: buffer, ``StringIO``
    :param numpy: numpy python module
    """
    try:
      _x = self
      buff.write(_get_struct_3I().pack(_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs))
      _x = self.header.frame_id
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      length = len(self.trajectories)
      buff.write(_struct_I.pack(length))
      for val1 in self.trajectories:
        buff.write(_get_struct_Q().pack(val1.track_id))
        length = len(val1.trajectory)
        buff.write(_struct_I.pack(length))
        for val2 in val1.trajectory:
          _v19 = val2.age
          _x = _v19
          buff.write(_get_struct_2i().pack(_x.secs, _x.nsecs))
          _x = val2
          buff.write(_get_struct_BQ().pack(_x.is_occluded, _x.detection_id))
          _v20 = val2.pose
          _v21 = _v20.pose
          _v22 = _v21.position
          _x = _v22
          buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
          _v23 = _v21.orientation
          _x = _v23
          buff.write(_get_struct_4d().pack(_x.x, _x.y, _x.z, _x.w))
          buff.write(_v20.covariance.tostring())
          _v24 = val2.twist
          _v25 = _v24.twist
          _v26 = _v25.linear
          _x = _v26
          buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
          _v27 = _v25.angular
          _x = _v27
          buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
          buff.write(_v24.covariance.tostring())
    except struct.error as se: self._check_types(struct.error("%s: '%s' when writing '%s'" % (type(se), str(se), str(locals().get('_x', self)))))
    except TypeError as te: self._check_types(ValueError("%s: '%s' when writing '%s'" % (type(te), str(te), str(locals().get('_x', self)))))

  def deserialize_numpy(self, str, numpy):
    """
    unpack serialized message in str into this message instance using numpy for array types
    :param str: byte array of serialized message, ``str``
    :param numpy: numpy python module
    """
    try:
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.trajectories is None:
        self.trajectories = None
      end = 0
      _x = self
      start = end
      end += 12
      (_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs,) = _get_struct_3I().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.header.frame_id = str[start:end].decode('utf-8')
      else:
        self.header.frame_id = str[start:end]
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      self.trajectories = []
      for i in range(0, length):
        val1 = frame_msgs.msg.PersonTrajectory()
        start = end
        end += 8
        (val1.track_id,) = _get_struct_Q().unpack(str[start:end])
        start = end
        end += 4
        (length,) = _struct_I.unpack(str[start:end])
        val1.trajectory = []
        for i in range(0, length):
          val2 = frame_msgs.msg.PersonTrajectoryEntry()
          _v28 = val2.age
          _x = _v28
          start = end
          end += 8
          (_x.secs, _x.nsecs,) = _get_struct_2i().unpack(str[start:end])
          _x = val2
          start = end
          end += 9
          (_x.is_occluded, _x.detection_id,) = _get_struct_BQ().unpack(str[start:end])
          val2.is_occluded = bool(val2.is_occluded)
          _v29 = val2.pose
          _v30 = _v29.pose
          _v31 = _v30.position
          _x = _v31
          start = end
          end += 24
          (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
          _v32 = _v30.orientation
          _x = _v32
          start = end
          end += 32
          (_x.x, _x.y, _x.z, _x.w,) = _get_struct_4d().unpack(str[start:end])
          start = end
          end += 288
          _v29.covariance = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=36)
          _v33 = val2.twist
          _v34 = _v33.twist
          _v35 = _v34.linear
          _x = _v35
          start = end
          end += 24
          (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
          _v36 = _v34.angular
          _x = _v36
          start = end
          end += 24
          (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
          start = end
          end += 288
          _v33.covariance = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=36)
          val1.trajectory.append(val2)
        self.trajectories.append(val1)
      return self
    except struct.error as e:
      raise genpy.DeserializationError(e) #most likely buffer underfill

_struct_I = genpy.struct_I
def _get_struct_I():
    global _struct_I
    return _struct_I
_struct_36d = None
def _get_struct_36d():
    global _struct_36d
    if _struct_36d is None:
        _struct_36d = struct.Struct("<36d")
    return _struct_36d
_struct_Q = None
def _get_struct_Q():
    global _struct_Q
    if _struct_Q is None:
        _struct_Q = struct.Struct("<Q")
    return _struct_Q
_struct_3I = None
def _get_struct_3I():
    global _struct_3I
    if _struct_3I is None:
        _struct_3I = struct.Struct("<3I")
    return _struct_3I
_struct_BQ = None
def _get_struct_BQ():
    global _struct_BQ
    if _struct_BQ is None:
        _struct_BQ = struct.Struct("<BQ")
    return _struct_BQ
_struct_4d = None
def _get_struct_4d():
    global _struct_4d
    if _struct_4d is None:
        _struct_4d = struct.Struct("<4d")
    return _struct_4d
_struct_2i = None
def _get_struct_2i():
    global _struct_2i
    if _struct_2i is None:
        _struct_2i = struct.Struct("<2i")
    return _struct_2i
_struct_3d = None
def _get_struct_3d():
    global _struct_3d
    if _struct_3d is None:
        _struct_3d = struct.Struct("<3d")
    return _struct_3d



from __future__ import print_function
import numpy as np
import os
from sensor_msgs.msg import LaserScan

from navrep.tools.data_extraction import archive_to_lidar_dataset
from navrep.tools.rings import generate_rings
from navrep.models.vae2d import ConvVAE, reset_graph

DEBUG_PLOTTING = True

# Parameters for training
batch_size = 1000
NUM_EPOCH = 100
DATA_DIR = "record"
HOME = os.path.expanduser("~")

vae_model_path = os.path.expanduser("~/navrep/models/V/vae.json")

# create network
reset_graph()
vae = ConvVAE(batch_size=batch_size, is_training=False)

# load
vae.load_json(vae_model_path)

# create training dataset
dataset = archive_to_lidar_dataset("~/navrep/datasets/V/ian", limit=180)
if len(dataset) == 0:
    raise ValueError("no scans found, exiting")
print(len(dataset), "scans in dataset.")
dataset = dataset[::100]

# split into batches:
total_length = len(dataset)
num_batches = int(np.floor(total_length / batch_size))

# rings converter
rings_def = generate_rings(64, 64)

dummy_msg = LaserScan()
dummy_msg.range_max = 100.0
dummy_msg.ranges = range(1080)

ring_accuracy_per_example = np.ones((len(dataset),)) * -1
for idx in range(num_batches):
    batch = dataset[idx * batch_size : (idx + 1) * batch_size]
    scans = batch
    rings = rings_def["lidar_to_rings"](scans).astype(float)

    obs = rings / rings_def["rings_to_bool"]
    # remove "too close" points
    obs[:, :, 0, :] = 0.0

    rings_pred = vae.encode_decode(obs) * rings_def["rings_to_bool"]
    returns = np.argmax(rings[:, :, :, :] > 0.4, axis=2).reshape((len(batch), 64))
    returns_pred = np.argmax(rings_pred[:, :, :, :] > 0.4, axis=2).reshape((len(batch), 64))
    ring_accuracy = np.sum(returns == returns_pred, axis=-1) / 64.
    ring_accuracy_per_example[idx * batch_size : (idx + 1) * batch_size] = ring_accuracy
    print("{:>4.1f}% - {:>4.3f}".format(
        idx * 100. / num_batches,
        np.mean(ring_accuracy_per_example[ring_accuracy_per_example != -1])
    ), end="\r")
dataset_mean_ring_accuracy = np.mean(ring_accuracy_per_example)
dataset_min_ring_accuracy = np.min(ring_accuracy_per_example)
dataset_max_ring_accuracy = np.max(ring_accuracy_per_example)
print(dataset_mean_ring_accuracy)
print(dataset_min_ring_accuracy)
print(dataset_max_ring_accuracy)



from pyniel.python_tools.path_tools import make_dir_if_not_exists
import time
import numpy as np
import os
from crowd_sim.envs.policy.orca import ORCA
from crowd_sim.envs.utils.state import JointState

from navrep.tools.commonargs import parse_multiproc_args
from navrep.tools.data_extraction import folder_to_archive
from navrep.envs.toyenv import ToyEnv
from navrep.envs.markenv import MarkEnv, FIRST_TRAIN_MAPS, SECOND_TRAIN_MAPS
from navrep.envs.navreptrainenv import NavRepTrainEnv
from navrep.tools.envplayer import EnvPlayer, WalltimeRate

class Suicide(object):
    def __init__(self):
        pass

class HumanControlPolicy(EnvPlayer):
    """ allows a human to provide the expert policy for dataset generation
    reuses the keyboard control functions from EnvPlayer """
    def __init__(self):
        self.render_mode = "human"
        self.STEP_BY_STEP = False
        self.reset()

    def reset(self):
        # variables
        self.action = np.array([0.0, 0.0, 0.0])
        self.restart = False
        self.exit = False
        self.action_key_is_set = False
        self.restart = False
        self.boost = False
        self.viewer = None
        self.realtime_rate = None

    def predict(self, obs, env):
        env.render(mode=self.render_mode)
        if self.viewer is None:
            self.viewer = env._get_viewer()
            self.viewer.window.on_key_press = self.key_press
            self.viewer.window.on_key_release = self.key_release
            self.realtime_rate = WalltimeRate(1.0 / env._get_dt())
        # synchronize (either with keypresses or walltime)
        if self.STEP_BY_STEP:
            # wait for keypress
            while True:
                if self.boost:
                    break
                if not self.action_key_is_set:
                    env.render(mode=self.render_mode)
                    time.sleep(0.01)
                else:
                    self.action_key_is_set = False
                    break
        else:
            if not self.boost:
                self.realtime_rate.sleep()
        return self.action

class SemiRandomMomentumPolicy(object):
    def __init__(self):
        self.speed = None

    def reset(self):
        self.speed = np.zeros((3,))

    def predict(self, obs, env):
        # simple heuristic - accelerate and face goal
        robotstate = obs[1]
        goal_xy = robotstate[:2]
        goal_dist = np.linalg.norm(goal_xy)
        best_xy = goal_xy / goal_dist if goal_dist != 0 else np.zeros_like(goal_xy)
        angle_to_goal = np.arctan2(goal_xy[1], goal_xy[0])
        best_rot = angle_to_goal * 0.1
        best_acc = np.array([best_xy[0], best_xy[1], best_rot])
        stddev = np.array([0.3, 0.3, 0.3])
        acceleration = np.random.normal(best_acc, stddev)
        friction = -self.speed * np.array([0.01, 0.01, 0.1])
        self.speed = np.clip(self.speed + acceleration + friction, -0.5, 0.5)
        return self.speed * 1.

class RandomMomentumPolicy(object):
    def __init__(self):
        self.speed = None

    def reset(self):
        self.speed = np.zeros((3,))

    def predict(self, obs, env):
        # create pseudo-random behavior
        acceleration = 0.1 * (np.random.random((3,)) - 0.5)
        friction = -self.speed * np.array([0.01, 0.01, 0.1])
        self.speed = np.clip(self.speed + acceleration + friction, -0.5, 0.5)
        return self.speed * 1.

class ORCAPolicy(object):
    def __init__(self, suicide_if_stuck=False):
        self.simulator = ORCA()
        self.suicide_if_stuck = suicide_if_stuck

    def reset(self):
        self.simulator.reset()

    def predict(self, obs, env):
        self.simulator.time_step = env._get_dt()
        other_agent_states = [
            agent.get_observable_state() for agent in env.soadrl_sim.humans + env.soadrl_sim.other_robots]
        action = self.simulator.predict(
            JointState(env.soadrl_sim.robot.get_full_state(), other_agent_states),
            env.soadrl_sim.obstacle_vertices,
            env.soadrl_sim.robot,
        )
        if self.suicide_if_stuck:
            if action.v < 0.1:
                return Suicide()
        vx = action.v * np.cos(action.r)
        vy = action.v * np.sin(action.r)
        return np.array([vx, vy, 0.1*(np.random.random()-0.5)])


def generate_vae_dataset(env, n_sequences,
                         episode_length=1000,
                         subset_index=0, n_subsets=1,
                         render=True,
                         policy=RandomMomentumPolicy(),
                         archive_dir=os.path.expanduser("~/navrep/datasets/V/ian")
                         ):
    """
    if n_subsets is None, the whole set of sequences is generated (n_sequences)
    if n_subsets is a number > 1, this function only generates a portion of the sequences
    """
    indices = np.arange(n_sequences)
    if n_subsets > 1:  # when multiprocessing
        indices = np.array_split(indices, n_subsets)[subset_index]
    for n in indices:
        scans = []
        robotstates = []
        actions = []
        rewards = []
        dones = []
        policy.reset()
        obs = env.reset()
        for i in range(episode_length):
            # step
            action = policy.predict(obs, env)
            if isinstance(action, Suicide):
                obs = env.reset()
                rew = 0
                action = np.array([0, 0, 0])
                done = True
            else:
                obs, rew, done, _ = env.step(action)
            scans.append(obs[0])
            robotstates.append(obs[1])
            actions.append(action)
            rewards.append(rew)
            dones.append(done)
            if render:
                env.render()
            if done:
                policy.reset()
                obs = env.reset()
            print("{} - {} {}".format(n, i, "done" if done else "     "), end="\r")
        dones[-1] = True

        scans = np.array(scans)
        robotstates = np.array(robotstates)
        actions = np.array(actions)
        rewards = np.array(rewards)
        dones = np.array(dones)
        data = dict(scans=scans, robotstates=robotstates, actions=actions, rewards=rewards, dones=dones)
        if archive_dir is not None:
            make_dir_if_not_exists(archive_dir)
            archive_path = os.path.join(
                archive_dir, "{:03}_scans_robotstates_actions_rewards_dones.npz".format(n)
            )
            np.savez_compressed(archive_path, **data)
            print(archive_path, "written.")
    env.close()
    return data


if __name__ == "__main__":
    args, _ = parse_multiproc_args()
    n_sequences = 100
    if args.n is not None:
        n_sequences = args.n

    if args.environment == "ian":
        # faster to use pre-existing ros bags than regenerate
        folder_to_archive("~/autoeval_bc")
    if args.environment == "toy":
        archive_dir = os.path.expanduser("~/navrep/datasets/V/toy")
        if args.dry_run:
            archive_dir = "/tmp/navrep/datasets/V/toy"
        env = ToyEnv(silent=True)
        generate_vae_dataset(
            env, n_sequences=n_sequences,
            subset_index=args.subproc_id, n_subsets=args.n_subprocs,
            render=args.render, archive_dir=archive_dir)
    if args.environment == "markone":
        archive_dir = os.path.expanduser("~/navrep/datasets/V/markone")
        if args.dry_run:
            archive_dir = "/tmp/navrep/datasets/V/markone"
        env = MarkEnv(silent=True, maps=FIRST_TRAIN_MAPS)
        generate_vae_dataset(
            env, n_sequences=n_sequences,
            subset_index=args.subproc_id, n_subsets=args.n_subprocs,
            render=args.render, archive_dir=archive_dir)
    if args.environment == "marktwo":
        archive_dir = os.path.expanduser("~/navrep/datasets/V/marktwo")
        if args.dry_run:
            archive_dir = "/tmp/navrep/datasets/V/marktwo"
        env = MarkEnv(silent=True, maps=SECOND_TRAIN_MAPS)
        generate_vae_dataset(
            env, n_sequences=n_sequences,
            subset_index=args.subproc_id, n_subsets=args.n_subprocs,
            render=args.render, archive_dir=archive_dir)
    if args.environment == "navreptrain":
        archive_dir = os.path.expanduser("~/navrep/datasets/V/navreptrain")
        if args.dry_run:
            archive_dir = "/tmp/navrep/datasets/V/navreptrain"
        env = NavRepTrainEnv(silent=True, scenario='train', adaptive=False, collect_statistics=False)
        env.soadrl_sim.human_num = 20
        generate_vae_dataset(
            env, n_sequences=n_sequences,
            subset_index=args.subproc_id, n_subsets=args.n_subprocs,
            policy=ORCAPolicy(suicide_if_stuck=True),
            render=args.render, archive_dir=archive_dir)
    if args.environment == "irl":
        folder_to_archive(
            directory="~/rosbags/iros_rosbags",
            archive_dir=os.path.expanduser("~/navrep/datasets/V/irl"),
        )



from __future__ import print_function
import numpy as np
import os
import rosbag
from tqdm import tqdm
from pose2d import Pose2D, apply_tf
from matplotlib import pyplot as plt
from CMap2D import CMap2D

from plot_ianenv_trajectories import blue, orange

DOWNSAMPLE = None  # 17 (1080 -> 64 rays)
OLD_FORMAT = False
ANIMATE = False

FIXED_FRAME = "gmap"
ROBOT_FRAME = "base_footprint"

GOAL_REACHED_DIST = 0.5


if __name__ == "__main__":
#     bag_path = os.path.expanduser("~/irl_tests/manip_corner_julian_jenjen.bag")
    bag_path = os.path.expanduser("~/irl_tests/hg_icra_round2.bag")

    cmdvel_topics = ["/cmd_vel"]
    joy_topics = ["/joy"]
    odom_topics = ["/pepper_robot/odom"]
    reward_topics = ["/patroller/reached_waypoint"]
    goal_topics = ["/move_base_simple/goal"]
    map_topics = ["/gmap"]
    topics = cmdvel_topics + joy_topics + odom_topics + reward_topics + goal_topics + map_topics

    bag_name = os.path.basename(bag_path)
    print("Loading {}...".format(bag_name))
    bag = rosbag.Bag(bag_path)
    try:
        import tf_bag
        bag_transformer = tf_bag.BagTfTransformer(bag)
    except ImportError:
        print("WARNING: Failed to import tf_bag. No goal information will be saved.")
        bag_transformer = None
        current_goal = [np.nan, np.nan]
    if bag.get_message_count(topic_filters=goal_topics) == 0:
        print("WARNING: No goal messages ({}) in rosbag. No goal information will be saved.".format(
            goal_topics))
        bag_transformer = None
        current_goal = [np.nan, np.nan]

    trajectories = []
    goals = []
    goals_reached = []
    trajectory = []
    goal = None
    i_frame = 0
    for topic, msg, t in tqdm(bag.read_messages(topics=topics),
                              total=bag.get_message_count(topic_filters=topics)):

        if topic in cmdvel_topics:
            cmd_vel = msg
            # store cmd_vel

        if topic in joy_topics:
            joy = msg
            # detect wether autonomous motion is active or not

        # process messages
        if topic in odom_topics:
            # velocity
            current_action = np.array([  # if msg is odometry
                msg.twist.twist.linear.x, msg.twist.twist.linear.y, msg.twist.twist.angular.z])

            # position
            try:
                p2_rob_in_fix = Pose2D(bag_transformer.lookupTransform(
                    FIXED_FRAME, ROBOT_FRAME, msg.header.stamp))
            except:  # noqa
                continue

            trajectory.append(p2_rob_in_fix)

        if topic in reward_topics:
            # goal is reached
            reached = True

        if topic in goal_topics:
            goal_in_msg = np.array([msg.pose.position.x, msg.pose.position.y])
            p2_msg_in_fix = Pose2D(bag_transformer.lookupTransform(
                FIXED_FRAME, msg.header.frame_id, msg.header.stamp))
            goal_in_fix = apply_tf(goal_in_msg[None, :], p2_msg_in_fix)[0]

            # end episode if goal change
            if goal is not None:
                if np.linalg.norm(goal - goal_in_fix) > GOAL_REACHED_DIST:
                    # goal change
                    trajectories.append(np.array(trajectory))
                    goals.append(goal)
                    goals_reached.append(np.any(
                        np.linalg.norm(np.array(trajectory)[:,:2] - goal, axis=1) < GOAL_REACHED_DIST))
                    trajectory = []

            # set goal
            goal = goal_in_fix

        if topic in map_topics:
            p2_map_in_fix = Pose2D(bag_transformer.lookupTransform(
                FIXED_FRAME, msg.header.frame_id, msg.header.stamp))
            mapmsg = msg

    fig, ax = plt.subplots(1, 1)

    if mapmsg is not None:
        map2d = CMap2D()
        map2d.from_msg(mapmsg)
        assert mapmsg.header.frame_id == FIXED_FRAME
        contours = map2d.as_closed_obst_vertices()
        for c in contours:
            cplus = np.concatenate((c, c[:1, :]), axis=0)
            ax.plot(cplus[:,0], cplus[:,1], color='k')
        plt.axis('equal')

    for t, g, s in zip(trajectories, goals, goals_reached):
        line_color = blue(len(t)/1000.)  # if s else orange(len(t)/1000.)
        zorder = 2 if s else 1
        if ANIMATE:
            yanim = np.ones_like(t[:,1]) * np.nan
            line, = ax.plot(t[:,0], yanim, color=line_color, zorder=zorder)
            ax.add_artist(plt.Circle((g[0], g[1]), 0.3, color="red", zorder=2))
            plt.pause(0.01)
            N = 10
            for i in range(0, len(yanim), N):
                yanim[i:i+N] = t[i:i+N,1]
                line.set_ydata(yanim)
                plt.pause(0.01)
                plt.savefig("/tmp/plot_irl_test_rosbags_{:05}.png".format(i_frame))
                i_frame += 1
        else:
            ax.plot(t[:,0], t[:,1], color=line_color, zorder=zorder)
            ax.add_artist(plt.Circle((g[0], g[1]), 0.3, color="red", zorder=2))
    ax.set_title(bag_name)
    ax.axis("equal")
    ax.set_adjustable('box')

    plt.show()



from gym.envs.box2d.car_racing import CarRacing
from tqdm import tqdm
import numpy as np

if __name__ == "__main__":
    env = CarRacing()
    env.reset()

    a = np.zeros(3)
    for _ in tqdm(range(1000000)):
        env.step(a)
#         env.render()



from __future__ import print_function
import numpy as np
import os
from sensor_msgs.msg import LaserScan

from navrep.tools.data_extraction import archive_to_lidar_dataset
from navrep.models.vae1d import Conv1DVAE, reset_graph

DEBUG_PLOTTING = True

# Parameters for training
batch_size = 1000
NUM_EPOCH = 100
DATA_DIR = "record"
HOME = os.path.expanduser("~")
MAX_LIDAR_DIST = 25.

vae_model_path = os.path.expanduser("~/navrep/models/V/vae1d.json")

# create network
reset_graph()
vae = Conv1DVAE(batch_size=batch_size, is_training=False)

# load
vae.load_json(vae_model_path)

# create training dataset
dataset = archive_to_lidar_dataset("~/navrep/datasets/V/ian", limit=180)
if len(dataset) == 0:
    raise ValueError("no scans found, exiting")
print(len(dataset), "scans in dataset.")
dataset = dataset[:500000:100]

# split into batches:
total_length = len(dataset)
num_batches = int(np.floor(total_length / batch_size))

dummy_msg = LaserScan()
dummy_msg.range_max = 100.0
dummy_msg.ranges = range(1080)

ring_accuracy_per_example = np.ones((len(dataset),)) * -1
for idx in range(num_batches):
    batch = dataset[idx * batch_size : (idx + 1) * batch_size]
    scans = batch

    obs = np.clip(scans.astype(np.float) / MAX_LIDAR_DIST, 0.0, MAX_LIDAR_DIST)
    obs = obs.reshape(batch_size, 1080, 1)

    obs_pred = vae.encode_decode(obs)
    error = abs(obs - obs_pred)
    threshold = np.minimum(0.001, abs(obs * 0.05))
    is_right = error <= threshold
    is_right = np.reshape(is_right, (batch_size, 1080))
    ring_accuracy = np.sum(is_right, axis=1) / 1080.
    ring_accuracy_per_example[idx * batch_size : (idx + 1) * batch_size] = ring_accuracy
    print("{:>4.1f}% - {:>4.3f}".format(
        idx * 100. / num_batches,
        np.mean(ring_accuracy_per_example[ring_accuracy_per_example != -1])
    ), end="\r")
if np.any(ring_accuracy_per_example == -1):
    raise ValueError("something weird.")
dataset_mean_ring_accuracy = np.mean(ring_accuracy_per_example)
dataset_min_ring_accuracy = np.min(ring_accuracy_per_example)
dataset_max_ring_accuracy = np.max(ring_accuracy_per_example)
print(dataset_mean_ring_accuracy)
print(dataset_min_ring_accuracy)
print(dataset_max_ring_accuracy)



from __future__ import print_function
import numpy as np
import json
import os
import matplotlib.pyplot as plt


env_name = "crowdnav"
optimizer = "cma"
num_rollouts = 16  # number of rollouts that are averaged over an episode
popsize = 32
required_score = 100.0


basedir = os.path.expanduser("~/navrep/models/cma")
logdirs = [x for x in os.listdir(basedir) if os.path.isdir(os.path.join(basedir, x))]
logdirs = sorted(logdirs)

all_data = []
all_best_data = []
titles = []
for logdir in logdirs:
    file_base = (
        env_name + "." + optimizer + "." + str(num_rollouts) + "." + str(popsize)
    )
    filename = os.path.join(basedir, logdir, file_base + ".hist.json")
    with open(filename, "r") as f:
        raw_data = json.load(f)
    data = np.array(raw_data)
    all_data.append(data)
    titles.append(filename)

    try:
        file_base = (
            env_name + "." + optimizer + "." + str(num_rollouts) + "." + str(popsize)
        )
        filename = os.path.join(basedir, logdir, file_base + ".hist_best.json")
        with open(filename, "r") as f:
            raw_data = json.load(f)
        raw_best_data = np.array(raw_data)
        best_data = []
        for bdata in raw_best_data:
            best_data.append(
                [
                    float(bdata[0]),
                    float(bdata[1]),
                    float(bdata[5]),
                    float(bdata[9]),
                    required_score,
                ]
            )
        best_data = np.array(best_data)
    except:
        best_data = np.zeros((10, 5)) * np.nan
    all_best_data.append(best_data)

    #     print(data)

    fig = plt.figure(figsize=(16, 10), dpi=80, facecolor="w", edgecolor="k")
    (line_mean,) = plt.plot(data[:, 1] / (60 * 24 * 60), data[:, 2])
    (line_min,) = plt.plot(data[:, 1] / (60 * 24 * 60), data[:, 3])
    (line_max,) = plt.plot(data[:, 1] / (60 * 24 * 60), data[:, 4])
    (line_best,) = plt.plot(best_data[:, 1] / (60 * 24 * 60), best_data[:, 2])
    (line_req,) = plt.plot(best_data[:, 1] / (60 * 24 * 60), best_data[:, 4])
    plt.axhline(
        color="k", zorder=-100,
    )
    plt.axhline(
        required_score, color="purple", zorder=-100,
    )
    plt.legend(
        [line_mean, line_min, line_max, line_req, line_best],
        ["mean", "min", "max", "requirement", "best avg score"],
    )
    plt.xlabel("wall-clock time (days)")
    plt.xticks(np.arange(0, 5, 1))
    plt.ylabel("cumulative reward")
    #     plt.yticks(np.arange(-100, 1000, 50))
    plt.title(filename)
    plt.show()

    fig = plt.figure(figsize=(16, 10), dpi=80, facecolor="w", edgecolor="k")
    (line_mean,) = plt.plot(data[:, 0], data[:, 2])
    (line_min,) = plt.plot(data[:, 0], data[:, 3])
    (line_max,) = plt.plot(data[:, 0], data[:, 4])
    (line_best,) = plt.plot(best_data[:, 0], best_data[:, 2])
    (line_req,) = plt.plot(best_data[:, 0], best_data[:, 4])
    plt.axhline(
        color="k", zorder=-100,
    )
    plt.axhline(
        required_score, color="purple", zorder=-100,
    )
    plt.legend(
        [line_mean, line_min, line_max, line_req, line_best],
        ["mean", "min", "max", "requirement", "best avg score"],
    )
    plt.xlabel("generation")
    #     plt.xticks(np.arange(0, 2000, 200))
    plt.ylabel("cumulative reward")
    #     plt.yticks(np.arange(-10, 1000, 10))
    plt.title(filename)
    plt.show()

fig = plt.figure(figsize=(16, 10), dpi=80, facecolor="w", edgecolor="k")
lines = []
legends = []
for data, best_data in zip(all_data, all_best_data):
    #     line, = plt.plot(data[:, 1]/(60*24*60), data[:, 2])
    #     line, = plt.plot(data[:, 1]/(60*24*60), data[:, 3])
    (line,) = plt.plot(data[:, 1] / (60 * 24 * 60), data[:, 4], alpha=0.6)
    lines.append(line)
    legends.append(filename)
    (line,) = plt.plot(
        best_data[:, 1] / (60 * 24 * 60),
        best_data[:, 2],
        alpha=0.6,
        color=line.get_color(),
    )
plt.axhline(
    color="k", zorder=-100,
)
plt.axhline(
    required_score, color="purple", zorder=-100,
)
plt.legend(lines, legends)
plt.xlabel("wall-clock time (days)")
plt.xticks(np.arange(0, 5, 1))
plt.ylabel("cumulative reward")
#     plt.yticks(np.arange(-100, 1000, 50))
# plt.title(filename)
plt.show()



from __future__ import print_function
import numpy as np
import os
import random
import time

from navrep.models.rnn import reset_graph, default_hps, MDNRNN, MAX_GOAL_DIST

_Z = 32

hps = default_hps()

hps = hps._replace(is_training=False)

dataset_folder = os.path.expanduser("~/navrep/datasets/M/toy")
model_hyperparams_path = os.path.expanduser("~/navrep/models/M/toyrnn.hyperparams.pckl")
model_path = os.path.expanduser("~/navrep/models/M/toyrnn.json")
vae_model_path = os.path.expanduser("~/navrep/models/V/toyvae.json")

# load preprocessed data
files = []
for dirpath, dirnames, filenames in os.walk(dataset_folder):
    for filename in [f for f in filenames if f.endswith(".npz")]:
        files.append(os.path.join(dirpath, filename))
all_data = []
for path in files:
    arrays = np.load(path)
    all_data.append(
        [
            arrays["mus"],
            arrays["logvars"],
            arrays["robotstates"],
            arrays["actions"],
            arrays["dones"],
            arrays["rewards"],
        ]
    )
n_total_frames = np.sum([mu.shape[0] for mu, _, _, _, _, _ in all_data])
print("total frames: ", n_total_frames)


reset_graph()
model = MDNRNN(hps)
model.load_json(model_path)

for epoch in range(1):
    epoch_z_costs = []
    epoch_wrongaction_z_costs = []
    #     print('preparing data for epoch', epoch)
    start = time.time()
    # flatten all sequences into one
    mu_sequence = np.zeros((n_total_frames, _Z), dtype=np.float32)
    logvar_sequence = np.zeros((n_total_frames, _Z), dtype=np.float32)
    robotstate_sequence = np.zeros((n_total_frames, 5), dtype=np.float32)
    action_sequence = np.zeros((n_total_frames, 3), dtype=np.float32)
    done_sequence = np.zeros((n_total_frames, 1), dtype=np.float32)
    reward_sequence = np.zeros((n_total_frames, 1), dtype=np.float32)
    i = 0
    random.shuffle(all_data)
    for mu, logvar, robotstate, action, done, reward in all_data:
        L = len(mu)
        mu_sequence[i : i + L, :] = mu.reshape(L, _Z)
        logvar_sequence[i : i + L, :] = logvar.reshape(L, _Z)
        robotstate_sequence[i : i + L, :] = robotstate.reshape(L, 5)
        action_sequence[i : i + L, :] = action.reshape(L, 3)
        done_sequence[i : i + L, :] = done.reshape(L, 1)
        reward_sequence[i : i + L, :] = reward.reshape(L, 1)
        i += L
    # sample z from  mu and logvar
    z_rs_sequence = mu_sequence + np.exp(logvar_sequence / 2.0) * np.random.randn(
        *(mu_sequence.shape)
    )
    # add goalstate (robotstate[:2]) to z
    robotstate_sequence[:, :2] = robotstate_sequence[:, :2] / MAX_GOAL_DIST  # normalize goal dist
    z_rs_sequence = np.concatenate([z_rs_sequence, robotstate_sequence[:, :2]], axis=-1)
    # resize array to be reshapable into sequences and batches
    chunksize = hps.batch_size * hps.max_seq_len  # frames per batch (100'000)
    n_chunks = n_total_frames // chunksize
    # reshape into sequences
    z_rs_sequences = np.reshape(
        z_rs_sequence[: n_chunks * chunksize, :], (-1, hps.max_seq_len, _Z)
    )
    action_sequences = np.reshape(
        action_sequence[: n_chunks * chunksize], (-1, hps.max_seq_len, 3)
    )
    done_sequences = np.reshape(
        done_sequence[: n_chunks * chunksize], (-1, hps.max_seq_len)
    )
    reward_sequences = np.reshape(
        reward_sequence[: n_chunks * chunksize], (-1, hps.max_seq_len)
    )
    num_sequences = len(z_rs_sequences)
    if num_sequences == 0:
        raise ValueError("Not enough data for a single batch")
    # shuffle
    random_idxs = list(range(num_sequences))
    random.shuffle(random_idxs)
    random_idxs = np.reshape(random_idxs, (-1, hps.batch_size))
    # reshape into batches
    z_rs_batches = z_rs_sequences[random_idxs]
    action_batches = action_sequences[random_idxs]
    done_batches = done_sequences[random_idxs]
    reward_batches = reward_sequences[random_idxs]
    num_batches = len(z_rs_batches)
    # result is of size (n_batches, batch_size, seq_len, ...)
    #     print('number of batches', num_batches)
    end = time.time()
    time_taken = end - start
    #     print('time taken to create batches', time_taken)

    batch_state = model.sess.run(model.initial_state)

    for batch_z_rs, batch_action, batch_done, batch_reward in zip(
        z_rs_batches, action_batches, done_batches, reward_batches
    ):

        feed = {
            model.batch_z_rs: batch_z_rs,
            model.batch_action: batch_action,
            model.batch_restart: batch_done,
            model.initial_state: batch_state,
        }
        z_cost = model.sess.run(model.z_cost, feed)
        feed = {
            model.batch_z_rs: batch_z_rs,
            model.batch_action: np.random.random(batch_action.shape),
            model.batch_restart: batch_done,
            model.initial_state: batch_state,
        }
        wrongaction_z_cost = model.sess.run(model.z_cost, feed)
        epoch_z_costs.append(z_cost)
        epoch_wrongaction_z_costs.append(wrongaction_z_cost)

print(np.mean(epoch_z_costs))
print(np.mean(epoch_wrongaction_z_costs))



#!/usr/bin/env python
from navrep.envs.dreamenv import DreamEnv
import os

from navrep.tools.envplayer import EnvPlayer

if __name__ == "__main__":
    os.environ["CUDA_VISIBLE_DEVICES"] = "-1"  # disable GPU

    denv = DreamEnv(temperature=0.)
    player = EnvPlayer(denv)



from __future__ import print_function
import time
import numpy as np
from pepper_2d_simulator import parse_PepperRLEnv_args, PepperRLEnv, check_PepperRLEnv_args
import os
import gym
from gym import spaces
from pandas import DataFrame
from pkg_resources import resource_filename

FIRST_TRAIN_MAPS = ["marktrain"]
FIRST_TEST_MAPS = ["marksimple", "markcomplex"]

SECOND_TRAIN_MAPS = ["marksimple", "markcomplex", "marktm1", "marktm2", "marktm3"]
SECOND_TEST_MAPS = ["markclutter", "markmaze"]

ALL_MAPS = FIRST_TRAIN_MAPS + SECOND_TRAIN_MAPS + SECOND_TEST_MAPS

DEFAULT_MAPS = SECOND_TEST_MAPS

class MarkEnv(gym.Env):
    """ This class wraps the IARLenv to encode the observation,
    resulting in z + h encoding instead of raw lidar scans.
    """
    metadata = {'render.modes': ['human']}

    def __init__(self, silent=False, maps=DEFAULT_MAPS, max_episode_length=1000, collect_trajectories=True):
        # gym env definition
        super(MarkEnv, self).__init__()
        self.action_space = spaces.Box(low=-1, high=1, shape=(3,), dtype=np.float32)
        self.observation_space = spaces.Tuple((
            spaces.Box(low=0, high=np.inf, shape=(1080,), dtype=np.float32),
            spaces.Box(low=0, high=np.inf, shape=(5,), dtype=np.float32),
        ))
        # constants
        self.COLLECT_TRAJECTORIES = collect_trajectories
        self.DT = 0.5  # should be the same as data rnn was trained with
        # parameters
        self.maps = maps
        self.silent = silent
        self.max_episode_length = max_episode_length
        self.max_episode_damage = 0
        # create underlying env
        self._make_env()
        # other tools
        self.episode_statistics = DataFrame(columns=[
            "total_steps",
            "scenario",
            "damage",
            "steps",
            "goal_reached",
            "timed_out",
            "collisioned_out",
            "goal",
            "trajectory",
            "reward",
            "wall_time",
        ])
        self.current_episode_trajectory = []
        self.current_episode_goal = None
        self.total_steps = 0
        # environment state variables
        self.reset()

    def step(self, action):
        self.steps_since_reset += 1
        self.total_steps += 1
        obs, reward, done, info = self.rlenv.step(action.reshape(1,3), ONLY_FOR_AGENT_0=True)
        timed_out = False
        collisioned_out = False
        if self.rlenv.episode_step[0] >= self.max_episode_length:
            done = True
            timed_out = True
        if self.rlenv.episode_damage[0] > 0:
            done = True
            collisioned_out = True
        if info["goal_reached"]:
            done = True
        # add to episodes log
        self.current_episode_trajectory.append(self.rlenv.virtual_peppers[0].pos * 1.)
        if done:
            traj = None
            if self.COLLECT_TRAJECTORIES:
                traj = np.array(self.current_episode_trajectory) * 1.
            self.episode_statistics.loc[len(self.episode_statistics)] = [
                self.total_steps,
                self.rlenv.args.map_name,
                self.rlenv.episode_damage[0] * 1.,
                self.steps_since_reset,
                info["goal_reached"],
                timed_out,
                collisioned_out,
                self.current_episode_goal * 1.,
                traj,
                reward,
                time.time(),
            ]
        obs = (obs[0][-1, :, 0], obs[1])  # latest scan only (buffer, ray, channel)
        return obs, reward, done, info

    def reset(self, set_scenario=None):
        if len(self.maps) > 1 or set_scenario is not None:
            self._make_env(set_scenario=set_scenario)
        obs = self.rlenv.reset(ONLY_FOR_AGENT_0=True)
        self.steps_since_reset = 0
        obs = (obs[0][-1, :, 0], obs[1])  # latest scan only (buffer, ray, channel)
        self.current_episode_trajectory = []
        self.current_episode_goal = self.rlenv.agent_goals[0] * 1.
        return obs

    def render(self, mode="human", close=False, lidar_scan_override=None, save_to_file=False):
        self.rlenv.render(mode=mode, close=close, lidar_scan_override=lidar_scan_override,
                          save_to_file=save_to_file)

    def close(self):
        try:
            self.rlenv.close()
        except AttributeError:
            pass

    def _get_viewer(self):
        viewer = None
        try:
            viewer = self.rlenv.viewer
        except AttributeError:
            pass
        return viewer

    def _make_env(self, set_scenario=None):
        try:
            self.rlenv.close()
        except AttributeError:
            pass
        args, _ = parse_PepperRLEnv_args(args=[])
        args.n_agents = 1
        map_name = set_scenario
        if map_name is None:
            map_name = np.random.choice(self.maps)
        args.map_name = map_name
        args.mode = "BOUNCE"
        args.continuous = True
        args.no_ros = True
        args.map_folder = resource_filename('asl_pepper_2d_sim_maps', 'maps')
        args.dt = self.DT
        check_PepperRLEnv_args(args)
        self.rlenv = PepperRLEnv(args, silent=self.silent)
        self.rlenv.virtual_peppers[0].NO_INERTIA = True

    def _get_dt(self):
        return self.rlenv.DT


if __name__ == "__main__":
    from navrep.tools.envplayer import EnvPlayer

    env = MarkEnv()
    player = EnvPlayer(env)



import os

from navrep.tools.envplayer import EnvPlayer
from navrep.envs.encodedenv import EncodedEnv
from navrep.tools.commonargs import parse_common_args

if __name__ == "__main__":
    args, _ = parse_common_args()

    env = EncodedEnv(args.backend, args.encoding)
    player = EnvPlayer(env)




# This Python file uses the following encoding: utf-8
"""autogenerated by genpy from frame_msgs/DetectedPersons.msg. Do not edit."""
import sys
python3 = True if sys.hexversion > 0x03000000 else False
import genpy
import struct

import frame_msgs.msg
import geometry_msgs.msg
import std_msgs.msg

class DetectedPersons(genpy.Message):
  _md5sum = "0b835e1ba9be8922e635c89b7b6640cb"
  _type = "frame_msgs/DetectedPersons"
  _has_header = True #flag to mark the presence of a Header object
  _full_text = """# Message with all currently detected persons
#

Header                  header          # Header containing timestamp etc. of this message
DetectedPerson[]        detections      # All persons that are currently being detected
================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.sec: seconds (stamp_secs) since epoch (in Python the variable is called 'secs')
# * stamp.nsec: nanoseconds since stamp_secs (in Python the variable is called 'nsecs')
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
string frame_id

================================================================================
MSG: frame_msgs/DetectedPerson
# Message describing a detection of a person
#

# Unique id of the detection, monotonically increasing over time
uint64          detection_id

# (Pseudo-)probabilistic value between 0.0 and 1.0 describing the detector's confidence in the detection
float64         confidence

# 3D pose (position + orientation) of the *center* of the detection
# check covariance to see which dimensions are actually set! unset dimensions shall have a covariance > 9999
geometry_msgs/PoseWithCovariance    pose

# height of detected person in 3D (in meters)
float64 height

# 2D bbox position (x and y is the top-left corner, width, height) 
float64 bbox_x
float64 bbox_y
float64 bbox_w
float64 bbox_h

# Sensor modality / detector type, see example constants below. 
# used e.g. later in tracking to check which tracks have been visually confirmed
string          modality            

                                    
string          MODALITY_GENERIC_LASER_2D = laser2d
string          MODALITY_GENERIC_LASER_3D = laser3d
string          MODALITY_GENERIC_MONOCULAR_VISION = mono
string          MODALITY_GENERIC_STEREO_VISION = stereo
string          MODALITY_GENERIC_RGBD = rgbd

# ReID output
float32[] embed_vector

================================================================================
MSG: geometry_msgs/PoseWithCovariance
# This represents a pose in free space with uncertainty.

Pose pose

# Row-major representation of the 6x6 covariance matrix
# The orientation parameters use a fixed-axis representation.
# In order, the parameters are:
# (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
float64[36] covariance

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of position and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w
"""
  __slots__ = ['header','detections']
  _slot_types = ['std_msgs/Header','frame_msgs/DetectedPerson[]']

  def __init__(self, *args, **kwds):
    """
    Constructor. Any message fields that are implicitly/explicitly
    set to None will be assigned a default value. The recommend
    use is keyword arguments as this is more robust to future message
    changes.  You cannot mix in-order arguments and keyword arguments.

    The available fields are:
       header,detections

    :param args: complete set of field values, in .msg order
    :param kwds: use keyword arguments corresponding to message field names
    to set specific fields.
    """
    if args or kwds:
      super(DetectedPersons, self).__init__(*args, **kwds)
      #message fields cannot be None, assign default values for those that are
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.detections is None:
        self.detections = []
    else:
      self.header = std_msgs.msg.Header()
      self.detections = []

  def _get_types(self):
    """
    internal API method
    """
    return self._slot_types

  def serialize(self, buff):
    """
    serialize message into buffer
    :param buff: buffer, ``StringIO``
    """
    try:
      _x = self
      buff.write(_get_struct_3I().pack(_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs))
      _x = self.header.frame_id
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      length = len(self.detections)
      buff.write(_struct_I.pack(length))
      for val1 in self.detections:
        _x = val1
        buff.write(_get_struct_Qd().pack(_x.detection_id, _x.confidence))
        _v1 = val1.pose
        _v2 = _v1.pose
        _v3 = _v2.position
        _x = _v3
        buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
        _v4 = _v2.orientation
        _x = _v4
        buff.write(_get_struct_4d().pack(_x.x, _x.y, _x.z, _x.w))
        buff.write(_get_struct_36d().pack(*_v1.covariance))
        _x = val1
        buff.write(_get_struct_5d().pack(_x.height, _x.bbox_x, _x.bbox_y, _x.bbox_w, _x.bbox_h))
        _x = val1.modality
        length = len(_x)
        if python3 or type(_x) == unicode:
          _x = _x.encode('utf-8')
          length = len(_x)
        buff.write(struct.pack('<I%ss'%length, length, _x))
        length = len(val1.embed_vector)
        buff.write(_struct_I.pack(length))
        pattern = '<%sf'%length
        buff.write(struct.pack(pattern, *val1.embed_vector))
    except struct.error as se: self._check_types(struct.error("%s: '%s' when writing '%s'" % (type(se), str(se), str(locals().get('_x', self)))))
    except TypeError as te: self._check_types(ValueError("%s: '%s' when writing '%s'" % (type(te), str(te), str(locals().get('_x', self)))))

  def deserialize(self, str):
    """
    unpack serialized message in str into this message instance
    :param str: byte array of serialized message, ``str``
    """
    try:
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.detections is None:
        self.detections = None
      end = 0
      _x = self
      start = end
      end += 12
      (_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs,) = _get_struct_3I().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.header.frame_id = str[start:end].decode('utf-8')
      else:
        self.header.frame_id = str[start:end]
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      self.detections = []
      for i in range(0, length):
        val1 = frame_msgs.msg.DetectedPerson()
        _x = val1
        start = end
        end += 16
        (_x.detection_id, _x.confidence,) = _get_struct_Qd().unpack(str[start:end])
        _v5 = val1.pose
        _v6 = _v5.pose
        _v7 = _v6.position
        _x = _v7
        start = end
        end += 24
        (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
        _v8 = _v6.orientation
        _x = _v8
        start = end
        end += 32
        (_x.x, _x.y, _x.z, _x.w,) = _get_struct_4d().unpack(str[start:end])
        start = end
        end += 288
        _v5.covariance = _get_struct_36d().unpack(str[start:end])
        _x = val1
        start = end
        end += 40
        (_x.height, _x.bbox_x, _x.bbox_y, _x.bbox_w, _x.bbox_h,) = _get_struct_5d().unpack(str[start:end])
        start = end
        end += 4
        (length,) = _struct_I.unpack(str[start:end])
        start = end
        end += length
        if python3:
          val1.modality = str[start:end].decode('utf-8')
        else:
          val1.modality = str[start:end]
        start = end
        end += 4
        (length,) = _struct_I.unpack(str[start:end])
        pattern = '<%sf'%length
        start = end
        end += struct.calcsize(pattern)
        val1.embed_vector = struct.unpack(pattern, str[start:end])
        self.detections.append(val1)
      return self
    except struct.error as e:
      raise genpy.DeserializationError(e) #most likely buffer underfill


  def serialize_numpy(self, buff, numpy):
    """
    serialize message with numpy array types into buffer
    :param buff: buffer, ``StringIO``
    :param numpy: numpy python module
    """
    try:
      _x = self
      buff.write(_get_struct_3I().pack(_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs))
      _x = self.header.frame_id
      length = len(_x)
      if python3 or type(_x) == unicode:
        _x = _x.encode('utf-8')
        length = len(_x)
      buff.write(struct.pack('<I%ss'%length, length, _x))
      length = len(self.detections)
      buff.write(_struct_I.pack(length))
      for val1 in self.detections:
        _x = val1
        buff.write(_get_struct_Qd().pack(_x.detection_id, _x.confidence))
        _v9 = val1.pose
        _v10 = _v9.pose
        _v11 = _v10.position
        _x = _v11
        buff.write(_get_struct_3d().pack(_x.x, _x.y, _x.z))
        _v12 = _v10.orientation
        _x = _v12
        buff.write(_get_struct_4d().pack(_x.x, _x.y, _x.z, _x.w))
        buff.write(_v9.covariance.tostring())
        _x = val1
        buff.write(_get_struct_5d().pack(_x.height, _x.bbox_x, _x.bbox_y, _x.bbox_w, _x.bbox_h))
        _x = val1.modality
        length = len(_x)
        if python3 or type(_x) == unicode:
          _x = _x.encode('utf-8')
          length = len(_x)
        buff.write(struct.pack('<I%ss'%length, length, _x))
        length = len(val1.embed_vector)
        buff.write(_struct_I.pack(length))
        pattern = '<%sf'%length
        buff.write(val1.embed_vector.tostring())
    except struct.error as se: self._check_types(struct.error("%s: '%s' when writing '%s'" % (type(se), str(se), str(locals().get('_x', self)))))
    except TypeError as te: self._check_types(ValueError("%s: '%s' when writing '%s'" % (type(te), str(te), str(locals().get('_x', self)))))

  def deserialize_numpy(self, str, numpy):
    """
    unpack serialized message in str into this message instance using numpy for array types
    :param str: byte array of serialized message, ``str``
    :param numpy: numpy python module
    """
    try:
      if self.header is None:
        self.header = std_msgs.msg.Header()
      if self.detections is None:
        self.detections = None
      end = 0
      _x = self
      start = end
      end += 12
      (_x.header.seq, _x.header.stamp.secs, _x.header.stamp.nsecs,) = _get_struct_3I().unpack(str[start:end])
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      start = end
      end += length
      if python3:
        self.header.frame_id = str[start:end].decode('utf-8')
      else:
        self.header.frame_id = str[start:end]
      start = end
      end += 4
      (length,) = _struct_I.unpack(str[start:end])
      self.detections = []
      for i in range(0, length):
        val1 = frame_msgs.msg.DetectedPerson()
        _x = val1
        start = end
        end += 16
        (_x.detection_id, _x.confidence,) = _get_struct_Qd().unpack(str[start:end])
        _v13 = val1.pose
        _v14 = _v13.pose
        _v15 = _v14.position
        _x = _v15
        start = end
        end += 24
        (_x.x, _x.y, _x.z,) = _get_struct_3d().unpack(str[start:end])
        _v16 = _v14.orientation
        _x = _v16
        start = end
        end += 32
        (_x.x, _x.y, _x.z, _x.w,) = _get_struct_4d().unpack(str[start:end])
        start = end
        end += 288
        _v13.covariance = numpy.frombuffer(str[start:end], dtype=numpy.float64, count=36)
        _x = val1
        start = end
        end += 40
        (_x.height, _x.bbox_x, _x.bbox_y, _x.bbox_w, _x.bbox_h,) = _get_struct_5d().unpack(str[start:end])
        start = end
        end += 4
        (length,) = _struct_I.unpack(str[start:end])
        start = end
        end += length
        if python3:
          val1.modality = str[start:end].decode('utf-8')
        else:
          val1.modality = str[start:end]
        start = end
        end += 4
        (length,) = _struct_I.unpack(str[start:end])
        pattern = '<%sf'%length
        start = end
        end += struct.calcsize(pattern)
        val1.embed_vector = numpy.frombuffer(str[start:end], dtype=numpy.float32, count=length)
        self.detections.append(val1)
      return self
    except struct.error as e:
      raise genpy.DeserializationError(e) #most likely buffer underfill

_struct_I = genpy.struct_I
def _get_struct_I():
    global _struct_I
    return _struct_I
_struct_36d = None
def _get_struct_36d():
    global _struct_36d
    if _struct_36d is None:
        _struct_36d = struct.Struct("<36d")
    return _struct_36d
_struct_3I = None
def _get_struct_3I():
    global _struct_3I
    if _struct_3I is None:
        _struct_3I = struct.Struct("<3I")
    return _struct_3I
_struct_Qd = None
def _get_struct_Qd():
    global _struct_Qd
    if _struct_Qd is None:
        _struct_Qd = struct.Struct("<Qd")
    return _struct_Qd
_struct_5d = None
def _get_struct_5d():
    global _struct_5d
    if _struct_5d is None:
        _struct_5d = struct.Struct("<5d")
    return _struct_5d
_struct_4d = None
def _get_struct_4d():
    global _struct_4d
    if _struct_4d is None:
        _struct_4d = struct.Struct("<4d")
    return _struct_4d
_struct_3d = None
def _get_struct_3d():
    global _struct_3d
    if _struct_3d is None:
        _struct_3d = struct.Struct("<3d")
    return _struct_3d



import numpy as np
import os
from pkg_resources import resource_filename

scenarios = [
    "irosasl1",
    "irosasl2",
    "irosasl3",
    "irosasl4",
    "irosasl5",
    "irosasl6",
    "irosasl_office_j1",
    "irosasl_office_j2",
    "irosasl_office_j3",
    "irosasl_office_j4",
    "irosasl_office_j5",
    "irosasl_office_j6",
    "irosunity_scene_map1",
    "irosunity_scene_map2",
    "irosunity_scene_map3",
    "irosunity_scene_map4",
    "irosunity_scene_map5",
    "irosunity_scene_map6",
]

scenario_maps = [
    "asl",
    "asl",
    "asl",
    "asl",
    "asl",
    "asl",
    "asl_office_j",
    "asl_office_j",
    "asl_office_j",
    "asl_office_j",
    "asl_office_j",
    "asl_office_j",
    "unity_scene_map",
    "unity_scene_map",
    "unity_scene_map",
    "unity_scene_map",
    "unity_scene_map",
    "unity_scene_map",
]

rl_scenarios = [
    "rlasl1",
    "rlasl2",
    "rlasl3",
    "rlasl_office_j1",
    "rlasl_office_j2",
    "rlasl_office_j3",
    "rlunity_scene_map1",
    "rlunity_scene_map2",
    "rlunity_scene_map3",
]

rl_scenario_maps = [
    "asl",
    "asl",
    "asl",
    "asl_office_j",
    "asl_office_j",
    "asl_office_j",
    "unity_scene_map",
    "unity_scene_map",
    "unity_scene_map",
]

# simplest curriculum
# rl_scenarios = [ "rlasl2", ]
# rl_scenario_maps = [ "asl", ]

# hard curriculum
rl_scenarios = scenarios + rl_scenarios
rl_scenario_maps = scenario_maps + rl_scenario_maps

map_downsampling = {
    "empty": 0,
    "asl": 1,
    "asl_office_j": 3,
    "unity_scene_map": 2,
    "crowdmove1": 0,
    "crowdmove2": 0,
    "crowdmove3": 0,
    "crowdmove4": 0,
    "crowdmove5": 0,
    "crowdmove6": 0,
}

# TODO: define episode length per scenario

def set_scenario(args, scenario_name=None):
    if scenario_name is None:
        scenario_id = np.random.randint(len(scenarios))
    else:
        scenario_id = get_scenario_id(scenario_name)
    args.map_folder = resource_filename('asl_pepper_2d_sim_maps', 'maps')
    args.scenario = scenarios[scenario_id]
    args.map_name = scenario_maps[scenario_id]
    args.map_downsampling_passes = map_downsampling[args.map_name]

def set_rl_scenario(args, scenario_name=None):
    if scenario_name is None:
        scenario_id = np.random.randint(len(rl_scenarios))
    else:
        scenario_id = get_rl_scenario_id(scenario_name)
    args.map_folder = resource_filename('asl_pepper_2d_sim_maps', 'maps')
    args.scenario = rl_scenarios[scenario_id]
    args.map_name = rl_scenario_maps[scenario_id]
    args.map_downsampling_passes = map_downsampling[args.map_name]

def get_scenario_id(scenario_name):
    for i, name in enumerate(scenarios):
        if name == scenario_name:
            return i
    return None

def get_rl_scenario_id(scenario_name):
    for i, name in enumerate(rl_scenarios):
        if name == scenario_name:
            return i
    return None



from __future__ import print_function
import numpy as np
import os

from navrep.tools.rings import generate_rings
from navrep.models.tcn import reset_graph, sample_hps_params, MDNTCN, get_pi_idx
from navrep.models.vae2d import ConvVAE

# parameters
TEMPERATURE = 0.5
_Z = 32

sequence_z_path = os.path.expanduser(
    "~/navrep/datasets/M/ian/000_mus_logvars_robotstates_actions_rewards_dones.npz"
)
rnn_model_path = os.path.expanduser("~/navrep/models/M/tcn.json")
vae_model_path = os.path.expanduser("~/navrep/models/V/vae.json")

reset_graph()
tcn = MDNTCN(sample_hps_params, gpu_mode=False)
vae = ConvVAE(batch_size=1, is_training=False)

vae.load_json(vae_model_path)
tcn.load_json(rnn_model_path)

rings_def = generate_rings(64, 64)

# load sequence image encoding
arrays = np.load(sequence_z_path)
sequence_action = arrays["actions"]
sequence_mu = arrays["mus"]
sequence_logvar = arrays["logvars"]
sequence_restart = arrays["dones"]
sequence_z = sequence_mu + np.exp(sequence_logvar / 2.0) * np.random.randn(
    *(sequence_mu.shape)
)


feed = {
    tcn.input_z: np.reshape(sequence_z[:999], (1, 999, _Z)),
    tcn.input_action: np.reshape(sequence_action[:999], (1, 999, 3)),
    tcn.input_restart: np.reshape(sequence_restart[:999], (1, 999)),
}

[logmix, mean, logstd, logrestart] = tcn.sess.run(
    [tcn.out_logmix, tcn.out_mean, tcn.out_logstd, tcn.out_restart_logits], feed
)

logmix = logmix.reshape((999, _Z, sample_hps_params.num_mixture))
mean = mean.reshape((999, _Z, sample_hps_params.num_mixture))
logstd = logstd.reshape((999, _Z, sample_hps_params.num_mixture))
logrestart = logrestart.reshape((999, 1))

OUTWIDTH = _Z

# adjust temperatures
logmix2 = np.copy(logmix) / TEMPERATURE
logmix2 -= logmix2.max()
logmix2 = np.exp(logmix2)
logmix2 /= logmix2.sum(axis=-1).reshape((999, _Z, 1))

mixture_idx = np.zeros((999, OUTWIDTH))
chosen_mean = np.zeros((999, OUTWIDTH))
chosen_logstd = np.zeros((999, OUTWIDTH))
for i in range(len(mixture_idx)):
    for j in range(OUTWIDTH):
        idx = get_pi_idx(np.random.rand(), logmix2[i, j])
        mixture_idx[i, j] = idx
        chosen_mean[i, j] = mean[i, j][idx]
        chosen_logstd[i, j] = logstd[i, j][idx]

rand_gaussian = np.random.randn(999, OUTWIDTH) * np.sqrt(TEMPERATURE)
next_z_predicted = chosen_mean + np.exp(chosen_logstd) * rand_gaussian
if sample_hps_params.differential_z:
    next_z_predicted = np.reshape(sequence_z[:999], (1, 999, _Z)) + next_z_predicted


rings_pred = vae.decode(sequence_z[:999].reshape(999, _Z)) * rings_def["rings_to_bool"]
rings_z_pred_pred = (
    vae.decode(next_z_predicted.reshape(999, _Z)) * rings_def["rings_to_bool"]
)
predicted_ranges = rings_def["rings_to_lidar"](rings_pred, 1080)

for i in range(len(rings_pred)):
    if True:
        import matplotlib.pyplot as plt

        plt.ion()
        plt.figure("rings")
        plt.clf()
        fig, (ax1, ax2) = plt.subplots(
            2, 1, subplot_kw=dict(projection="polar"), num="rings"
        )
        rings_def["visualize_rings"](rings_pred[i], scan=None, fig=fig, ax=ax1)
        rings_def["visualize_rings"](rings_z_pred_pred[i], scan=None, fig=fig, ax=ax2)
        ax1.set_ylim([0, 10])
        ax1.set_title(i)
        ax2.set_ylim([0, 10])
        # update
        plt.pause(0.01)



import os
import numpy as np
from tqdm import tqdm
from pyniel.python_tools.path_tools import make_dir_if_not_exists

from navrep.tools.commonargs import parse_common_args
from navrep.envs.encodedenv import EncodedEnv
from navrep.envs.navreptrainencodedenv import NavRepTrainEncoder
from navrep.envs.e2eenv import E2EIANEnv, E2E1DIANEnv
from navrep.scripts.test_navrep import NavRepCPolicy
from navrep.scripts.test_e2e import E2ECPolicy, E2E1DCPolicy

def run_test_episodes(env, policy, render=False, num_episodes=1000):
    progress_bar = tqdm(range(num_episodes), total=num_episodes)
    for i in progress_bar:
        live_success = np.mean(env.episode_statistics["goal_reached"])
        ob = env.reset()
        done = False
        env_time = 0
        while not done:
            action = policy.act(ob)
            ob, _, done, info = env.step(action)
            if render:
                env.render('human')  # , save_to_file=True)
            env_time += env._get_dt()
        progress_bar.set_description("Cross Testing: Episode {} - {:.2f}".format(i, live_success))
        if i % 1000 == 0:
            path = "/tmp/cross_test_X_in_X.csv"
            env.episode_statistics.to_csv(path)
            print("{} written.".format(path))

    S = env.episode_statistics

    all_success_rate = np.mean(S["goal_reached"])
    print("ALL: {:.1f}%".format(100.*all_success_rate))
    print("-----------")

    scenarios = np.array(sorted(list(set(S["scenario"]))))
    for scenario in scenarios:
        scenario_specific = S[S["scenario"] == scenario]
        success_rate = np.mean(scenario_specific["goal_reached"])
        print("{}: {}%".format(scenario, 100.*success_rate))

    return S


# run xtest on all stored controllers for the desired backend/encoding combo
if __name__ == '__main__':
    from stable_baselines import PPO2

    args, _ = parse_common_args()

    # defaults
    if args.n is None:
        args.n = 1000
    collect_trajectories = True

    # find compatible c models
    gym_dir = os.path.expanduser("~/navrep/models/gym")
    candidates = sorted(os.listdir(gym_dir))
    compatible = [name for name in candidates if args.backend + "_" + args.encoding in name]
    if not compatible:
        print(candidates)
        print("Compatible C model for backend and encoding not found in candidates")
        raise ValueError
    print("Compatible C models:")
    for name in compatible:
        print(name)
    print()

    # run cross test for each compatible c model
    progress_bar = tqdm(compatible)
    for c_model_name in progress_bar:
        c_model_info = c_model_name.split("_ckpt.zip")[0]
        c_model_path = os.path.join(gym_dir,c_model_name)
        print("C Model '{}' selected".format(c_model_path))

        if "E2E1D" in args.backend:
            # this is necessary because the pickled file expects a module called custom_policy
            import sys
            import navrep.tools.custom_policy
            sys.modules['custom_policy'] = navrep.tools.custom_policy
            # env and policy
            env = E2E1DIANEnv(silent=True, collect_trajectories=collect_trajectories)
            policy = E2E1DCPolicy(c_model_path)
        elif "E2E" in args.backend:
            # this is necessary because the pickled file expects a module called custom_policy
            import sys
            import navrep.tools.custom_policy
            sys.modules['custom_policy'] = navrep.tools.custom_policy
            # env and policy
            env = E2EIANEnv(silent=True, collect_trajectories=collect_trajectories)
            policy = E2ECPolicy(c_model_path)
        else:
            # create ianenv with soadrl encoder
            encoder = NavRepTrainEncoder(args.backend, args.encoding, gpu=not args.no_gpu)
            env = EncodedEnv(None, None, silent=True, encoder=encoder,
                             collect_trajectories=collect_trajectories)

            # gym model
            c_model = PPO2.load(c_model_path)
            policy = NavRepCPolicy(c_model)

        progress_bar.set_description("C model strain")
        S = run_test_episodes(env, policy, render=args.render, num_episodes=args.n)

        DIR = os.path.expanduser("~/navrep/eval/crosstest")
        if args.dry_run:
            DIR = "/tmp/navrep/eval/crosstest"
        make_dir_if_not_exists(DIR)
        if collect_trajectories:
            NAME = "{}_in_ianenv_x{}.pckl".format(c_model_info, len(S))
            PATH = os.path.join(DIR, NAME)
            S.to_pickle(PATH)
        else:
            NAME = "{}_in_ianenv_x{}.csv".format(c_model_info, len(S))
            PATH = os.path.join(DIR, NAME)
            S.to_csv(PATH)
        print("{} written.".format(PATH))
